@@ -37,12 +37,12 @@
 #include <locale.h>
 #include <signal.h>
 
-#if WINDOWS || OSX
-#ifdef LOCALEDIR
-#undef LOCALEDIR
-#endif	/* LOCALEDIR */
-char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *PLUGINDIR, *CONFDIR, *LOCALEDIR, *BINDIR;
-#endif	/* WINDOWS */
+#if OSX
+#include <gtkosxapplication.h>
+static GtkosxApplication *GtkMacIntegration = NULL;
+#endif
+
+char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *PLUGINDIR, *CONFDIR, *NLOCALEDIR, *BINDIR;
 
 #include "dir_defs.h"
 #include "object.h"
@@ -50,6 +50,7 @@ char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *PLUGINDIR, *CONFDIR, *LOCALEDIR, *
 #include "nstring.h"
 #include "nconfig.h"
 #include "shell.h"
+#include "gtk_action.h"
 #include "init.h"
 
 #ifdef HAVE_READLINE_READLINE_H
@@ -59,6 +60,7 @@ char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *PLUGINDIR, *CONFDIR, *LOCALEDIR, *
 #include "x11menu.h"
 #include "ogra2x11.h"
 #include "ogra2cairo.h"
+#include "x11commn.h"
 #include <assert.h>
 static char **attempt_shell_completion(char *text, int start, int end);
 char *HistoryFile = NULL;
@@ -66,6 +68,8 @@ char *HistoryFile = NULL;
 #define HIST_FILE "shell_history"
 #endif	/* HAVE_READLINE_READLINE_H */
 
+GtkApplication *GtkApp = NULL;
+
 #define CSS_PATH RESOURCE_PATH "/css/ngraph.css"
 #define SYSCONF "[Ngraph]"
 
@@ -108,6 +112,7 @@ void *addaxis(void);
 void *addagrid(void);
 void *addprm(void);
 void *addpath(void);
+void *addparameter(void);
 
 void *addgra2gtk(void);
 void *addmenu(void);
@@ -120,6 +125,9 @@ void resizeconsole(int col, int row);
 
 // XtAppContext Application=NULL;
 char *AppName = "Ngraph", *AppClass = "Ngraph", *Home;
+#if ! OSX
+static const char *Snap = NULL;
+#endif
 
 char *Auther[] = {
   "Satoshi ISHIZAKA",
@@ -175,6 +183,7 @@ static void * ( * obj_add_func_ary[]) (void) = {
   addpath,
   addmark,
   addtext,
+  addparameter,
   addmenu,
   adddialog,
 #if WINDOWS
@@ -384,7 +393,6 @@ resotre_console(void)
 static int
 exec_console(char *fifo_in, char *fifo_out)
 {
-  char **argv;
   pid_t pid;
 
   pid = fork();
@@ -395,17 +403,22 @@ exec_console(char *fifo_in, char *fifo_out)
   } else if (pid == 0) {
     pid = fork();
     if (pid == 0) {
-      char buf[2049], *s2, *s;
-      int len;
+      char buf[2049];
 #if OSX
       snprintf(buf, sizeof(buf), "%s -e %s/terminal %s %s", Terminal, LIBDIR, fifo_in, fifo_out);
       system(buf);
       exit(0);
 #else
+      char **argv, *s2, *s;
+      int len;
       snprintf(buf, sizeof(buf), "%s %s %s", Terminal, fifo_in, fifo_out);
       argv = NULL;
       s = buf;
       while ((s2 = getitok2(&s, &len, " \t")) != NULL) {
+	if (Snap && g_str_has_suffix(s2, "/terminal")) {
+	  g_free(s2);
+	  s2 = g_strdup_printf("%s/terminal", LIBDIR);
+	}
 	arg_add(&argv, s2);
       }
       execvp(argv[0], argv);
@@ -624,62 +637,64 @@ load_config(struct objlist *sys, N_VALUE *inst, int *allocconsole)
   int len;
   long val;
 
-  if ((fp = openconfig(SYSCONF)) != NULL) {
-    while ((tok = getconfig(fp, &str)) != NULL) {
-      s2 = str;
-      if (strcmp(tok, "login_shell") == 0) {
-	f1 = getitok2(&s2, &len, " \t,");
-	if (_putobj(sys, "login_shell", inst, f1))
-	  exit(1);
-      } else if (strcmp(tok, "create_object") == 0) {
-	while ((f1 = getitok2(&s2, &len, " \t,")) != NULL) {
-	  struct objlist *obj;
-	  obj = getobject(f1);
-	  if (obj) {
-	    newobj(obj);
-	  }
-	  g_free(f1);
-	}
-      } else if (strcmp(tok, "alloc_console") == 0) {
-	f1 = getitok2(&s2, &len, " \t,");
-	val = strtol(f1, &endptr, 10);
-	if (endptr[0] == '\0') {
-	  if (val == 0)
-	    *allocconsole = FALSE;
-	  else
-	    *allocconsole = TRUE;
-	}
-	g_free(f1);
-      } else if (strcmp(tok, "console_size") == 0) {
-	f1 = getitok2(&s2, &len, " \x09,");
-	val = strtol(f1, &endptr, 10);
-	if (endptr[0] == '\0')
-	  consolecol = val;
-	g_free(f1);
-	f1 = getitok2(&s2, &len, " \x09,");
-	val = strtol(f1, &endptr, 10);
-	if (endptr[0] == '\0')
-	  consolerow = val;
-	g_free(f1);
-#ifdef HAVE_READLINE_READLINE_H
-      } else if (strcmp(tok, "history_size") == 0) {
-	f1 = getitok2(&s2, &len, " \t,");
-	val = strtol(f1, &endptr, 10);
-	if (endptr[0] == '\0' && val > 0) {
-	  *history_size = val;
-	}
-	g_free(f1);
-#endif
-      } else if (strcmp(tok, "terminal") == 0) {
-	Terminal = getitok2(&s2, &len, "");
-      } else {
-	fprintf(stderr, "(%s): configuration '%s' in section %s is not used.\n", AppName, tok, SYSCONF);
-      }
-      g_free(tok);
-      g_free(str);
-    }
-    closeconfig(fp);
+  fp = openconfig(SYSCONF);
+  if (fp == NULL) {
+    return;
   }
+  while ((tok = getconfig(fp, &str)) != NULL) {
+    s2 = str;
+    if (strcmp(tok, "login_shell") == 0) {
+      f1 = getitok2(&s2, &len, " \t,");
+      if (_putobj(sys, "login_shell", inst, f1))
+        exit(1);
+    } else if (strcmp(tok, "create_object") == 0) {
+      while ((f1 = getitok2(&s2, &len, " \t,")) != NULL) {
+        struct objlist *obj;
+        obj = getobject(f1);
+        if (obj) {
+          newobj(obj);
+        }
+        g_free(f1);
+      }
+    } else if (strcmp(tok, "alloc_console") == 0) {
+      f1 = getitok2(&s2, &len, " \t,");
+      val = strtol(f1, &endptr, 10);
+      if (endptr[0] == '\0') {
+        if (val == 0)
+          *allocconsole = FALSE;
+        else
+          *allocconsole = TRUE;
+      }
+      g_free(f1);
+    } else if (strcmp(tok, "console_size") == 0) {
+      f1 = getitok2(&s2, &len, " \x09,");
+      val = strtol(f1, &endptr, 10);
+      if (endptr[0] == '\0')
+        consolecol = val;
+      g_free(f1);
+      f1 = getitok2(&s2, &len, " \x09,");
+      val = strtol(f1, &endptr, 10);
+      if (endptr[0] == '\0')
+        consolerow = val;
+      g_free(f1);
+#ifdef HAVE_READLINE_READLINE_H
+    } else if (strcmp(tok, "history_size") == 0) {
+      f1 = getitok2(&s2, &len, " \t,");
+      val = strtol(f1, &endptr, 10);
+      if (endptr[0] == '\0' && val > 0) {
+        *history_size = val;
+      }
+      g_free(f1);
+#endif
+    } else if (strcmp(tok, "terminal") == 0) {
+      Terminal = getitok2(&s2, &len, "");
+    } else {
+      fprintf(stderr, "(%s): configuration '%s' in section %s is not used.\n", AppName, tok, SYSCONF);
+    }
+    g_free(tok);
+    g_free(str);
+  }
+  closeconfig(fp);
 }
 
 #if WINDOWS
@@ -688,12 +703,15 @@ set_dir_defs(char *app)
 {
   char *utf8_name, *app_path, *tmp;
 
-  utf8_name = g_locale_to_utf8(app, -1, NULL, NULL, NULL);
+  utf8_name = n_locale_to_utf8(app);
   if (utf8_name == NULL) {
     return 1;
   }
 
   changefilename(utf8_name);
+  tmp = g_canonicalize_filename(utf8_name, NULL);
+  g_free(utf8_name);
+  utf8_name = tmp;
 
   tmp = g_path_get_dirname(utf8_name);
   g_free(utf8_name);
@@ -714,30 +732,29 @@ set_dir_defs(char *app)
   NDATADIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk");
   ADDINDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk/addin");
   CONFDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "etc/ngraph-gtk");
-  LOCALEDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/locale");
+  NLOCALEDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/locale");
 
   g_free(app_path);
 
   return 0;
 }
-#endif	/* WINDOWS */
-
-#if OSX
+#elif OSX
 static int
 set_dir_defs(char *app)
 {
   const char *app_contents;
-  char *app_path;
+  char *app_path, *bin_path;
 
   app_contents = g_getenv("NGRAPH_APP_CONTENTS");
+  app_path = gtkosx_application_get_resource_path();
   if (app_contents) {
-    app_path = g_strdup_printf("%s%c%s", app_contents, DIRSEP, "Resources");
-    LIBDIR = g_strdup_printf("%s%c%s", app_contents, DIRSEP, "MacOS");
-  } else {
-    char *bin_path;
-    bin_path = g_path_get_dirname(app);
-    app_path = g_path_get_dirname(bin_path);
+    bin_path = gtkosx_application_get_executable_path();
+    LIBDIR = g_path_get_dirname(bin_path);
     g_free(bin_path);
+  } else {
+    bin_path = g_path_get_dirname(app_path);
+    g_free(app_path);
+    app_path = bin_path;
     LIBDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "libexec/ngraph-gtk");
   }
 
@@ -751,22 +768,48 @@ set_dir_defs(char *app)
   NDATADIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk");
   ADDINDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk/addin");
   CONFDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "etc/ngraph-gtk");
-  LOCALEDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/locale");
+  NLOCALEDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/locale");
 
   g_free(app_path);
 
   return 0;
 }
+#else
+static int
+set_dir_defs(char *app)
+{
+  const char *snap;
+
+  snap = g_getenv("SNAP");
+  if (snap) {
+    BINDIR = g_strdup_printf("%s%c%s", snap, DIRSEP, "bin");
+    Snap = snap;
+  } else {
+    BINDIR = NULL;
+    snap = "";
+  }
+
+  DOCDIR = g_strdup_printf("%s%s", snap, _DOCDIR);
+  LIBDIR = g_strdup_printf("%s%s", snap, _LIBDIR);
+  PLUGINDIR = g_strdup_printf("%s%s", snap, _PLUGINDIR);
+  NDATADIR = g_strdup_printf("%s%s", snap, _NDATADIR);
+  ADDINDIR = g_strdup_printf("%s%s", snap, _ADDINDIR);
+  CONFDIR = g_strdup_printf("%s%s", snap, _CONFDIR);
+  NLOCALEDIR = g_strdup_printf("%s%s", snap, LOCALEDIR);
+
+  return 0;
+}
 #endif
 
-
-#if WINDOWS
 static void
 set_path_env(char *homedir)
 {
   const char *path;
   char *pathset;
 
+  if (BINDIR == NULL) {
+    return;
+  }
   path = g_getenv("PATH");
   pathset = g_strdup_printf("%s%s%s%s%s%s%s%s%s%s%s",
 			    homedir, PATHSEP,
@@ -781,7 +824,6 @@ set_path_env(char *homedir)
   g_setenv("NGRAPHCONF", CONFDIR, TRUE);
   g_free(pathset);
 }
-#endif
 
 const char *
 n_getlocale(void)
@@ -801,6 +843,46 @@ load_css(void)
 }
 #endif
 
+void
+n_application_ready(void)
+{
+#if OSX
+  if (GtkMacIntegration) {
+    gtkosx_application_ready(GtkMacIntegration);
+  }
+#endif
+}
+
+#if OSX
+static gboolean
+osx_open_file(GtkosxApplication *app, gchar *path, gpointer user_data)
+{
+  char *dir;
+  if (path == NULL) {
+    return TRUE;
+  }
+  dir = getdirname(path);
+  if (dir) {
+    nchdir(dir);
+    g_free(dir);
+  }
+  LoadNgpFile(path, FALSE, "-f");
+  return TRUE;
+}
+
+static void
+create_app_menu(GtkApplication *app)
+{
+  GtkBuilder *builder;
+  GObject *app_menu;
+  /* only for remove "Settings" menu item */
+  builder = gtk_builder_new_from_resource(RESOURCE_PATH "/gtk/menus-appmenu.ui");
+  app_menu = gtk_builder_get_object(builder, "app-menu");
+  gtk_application_set_app_menu(app, G_MENU_MODEL(app_menu));
+  g_object_unref(builder);
+}
+#endif
+
 int
 n_initialize(int *argc, char ***argv)
 {
@@ -818,10 +900,6 @@ n_initialize(int *argc, char ***argv)
     return 0;
   }
 
-#if ! GLIB_CHECK_VERSION(2, 32, 0)
-  g_thread_init(NULL);
-#endif
-
 #if EOF == -1
   char_type_buf_init();
 #endif
@@ -830,12 +908,29 @@ n_initialize(int *argc, char ***argv)
   set_childhandler();
 #endif	/* WINDOWS */
 
-  OpenDisplay = gtk_init_check(argc, argv);
-  g_set_application_name(AppName);
-
-#if WINDOWS || OSX
   set_dir_defs((*argv)[0]);
-#endif	/* WINDOWS */
+
+#ifdef HAVE_GETTEXT
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, NLOCALEDIR);
+  bind_textdomain_codeset(PACKAGE, "UTF-8");
+  textdomain(PACKAGE);
+#endif	/* HAVE_GETTEXT */
+
+  OpenDisplay = gtk_init_check(argc, argv);
+  if (OpenDisplay) {
+    GtkApp = gtk_application_new(APPLICATION_ID, G_APPLICATION_NON_UNIQUE);
+    g_application_register(G_APPLICATION(GtkApp), NULL, NULL);
+    setup_actions(GtkApp);
+  }
+  g_set_application_name(AppName);
+#if OSX
+  GtkMacIntegration = gtkosx_application_get();
+  g_signal_connect(GtkMacIntegration, "NSApplicationOpenFile", G_CALLBACK(osx_open_file), NULL);
+  if (OpenDisplay) {
+    create_app_menu(GtkApp);
+  }
+#endif
 
   if (init_cmd_tbl()) {
     exit(1);
@@ -855,13 +950,6 @@ n_initialize(int *argc, char ***argv)
   }
 #endif	/* GTK_CHECK_VERSION(3, 16, 0) */
 
-#ifdef HAVE_GETTEXT
-  setlocale(LC_ALL, "");
-  bindtextdomain(PACKAGE, LOCALEDIR);
-  bind_textdomain_codeset(PACKAGE, "UTF-8");
-  textdomain(PACKAGE);
-#endif	/* HAVE_GETTEXT */
-
 #if WINDOWS
   locale = g_win32_getlocale();
 #else  /* WINDOWS */
@@ -911,9 +999,7 @@ n_initialize(int *argc, char ***argv)
     }
   }
 
-#if WINDOWS
   set_path_env(homedir);
-#endif
   set_environ();
 
   if (addobjectroot() == NULL)
@@ -1101,7 +1187,7 @@ attempt_shell_completion(char *text, int start, int end)
     in_command_position++;
   }
 
-  if (!matches && in_command_position)
+  if (in_command_position)
     matches = rl_completion_matches(text, command_word_completion_function);
 
   if (!matches)
@@ -1118,11 +1204,11 @@ obj_member_completion_function(const char *text, int state)
 {
   static char **list = (char **) NULL;
   static int list_index = 0, first_char_loc;
-  struct objlist *objcur;
 
   /* If we don't have any state, make some. */
   if (!state) {
     static char *obj, *instances, *member, *val;
+    struct objlist *objcur;
 
     if (list)
       g_free(list);
