@@ -161,8 +161,10 @@ get_char_set(gunichar ch)
     char_set = DEFAULT_CHARSET;
     break;
   case G_UNICODE_SCRIPT_HAN:
+/*
     char_set = CHINESEBIG5_CHARSET;
     char_set = GB2312_CHARSET;
+*/
     char_set = SHIFTJIS_CHARSET;
     break;
   case G_UNICODE_SCRIPT_HANGUL:
@@ -897,7 +899,7 @@ select_font(struct gra2emf_local *local, const char *fontname, int charset)
   }
 
   id_font.lfCharSet = charset;
-  id_font.lfOutPrecision = OUT_DEFAULT_PRECIS;
+//  id_font.lfOutPrecision = OUT_DEFAULT_PRECIS;
   id_font.lfOutPrecision = OUT_TT_ONLY_PRECIS;
   id_font.lfClipPrecision = CLIP_STROKE_PRECIS;
   id_font.lfQuality = PROOF_QUALITY;
@@ -1040,13 +1042,14 @@ draw_str(struct gra2emf_local *local, const char *str)
   const char *ptr;
   GString *sub_str;
   const char *font, *prev_font;
-  int charset, prev_charset;
+  int prev_charset;
 
   sub_str = g_string_new("");
   prev_font = NULL;
   prev_charset = DEFAULT_CHARSET;
 
   for (ptr = str; *ptr; ptr = g_utf8_next_char(ptr)) {
+    int charset;
     ch = g_utf8_get_char(ptr);
     charset = get_char_set(ch);
     font = check_font_indices(local, ch);
@@ -1495,10 +1498,9 @@ static int
 gra2emf_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char code, *cstr, *tmp, *fname;
-  int *cpar, i, r;
+  int *cpar, r;
   double x, y, w, h, fontdir;
   struct gra2emf_local *local;
-  DWORD *line_style;
   POINT lpoint[2];
 
   local = (struct gra2emf_local *)argv[2];
@@ -1557,7 +1559,8 @@ gra2emf_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
     local->line_style = NULL;
     local->line_style_num = 0;
     if (cpar[1]) {
-      local->line_style = g_malloc(sizeof(* line_style) * cpar[1]);
+      int i;
+      local->line_style = g_malloc(sizeof(* (local->line_style)) * cpar[1]);
       if (local->line_style == NULL) {
 	break;
       }
