@@ -108,14 +108,12 @@ enum menu_config_type {
   MENU_CONFIG_TYPE_COLOR,
   MENU_CONFIG_TYPE_COLOR_ARY,
   MENU_CONFIG_TYPE_SCRIPT,
-  MENU_CONFIG_TYPE_DRIVER,
   MENU_CONFIG_TYPE_CHARMAP,
 };
 
 static int menu_config_set_custom_palette(char *s2, void *data);
 static int menu_config_set_four_elements(char *s2, void *data);
 static int menu_config_set_bgcolor(char *s2, void *data);
-static int menu_config_set_ext_driver(char *s2, void *data);
 static int menu_config_set_script(char *s2, void *data);
 static int menu_config_set_char_map(char *s2, void *data);
 
@@ -138,11 +136,7 @@ static struct menu_config MenuConfig[] = {
   {"addin_console",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.addinconsole},
   {"show_tip",			MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.showtip},
   {"character_map",		MENU_CONFIG_TYPE_CHARMAP, menu_config_set_char_map, &Menulocal.char_map},
-  {NULL},
-};
-
-static struct menu_config MenuConfigDriver[] = {
-  {"ext_driver", MENU_CONFIG_TYPE_DRIVER, menu_config_set_ext_driver, NULL},
+  {"help_file",			MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.help_file},
   {NULL},
 };
 
@@ -207,7 +201,6 @@ static struct menu_config MenuConfigOthers[] = {
 #endif
   {"ps_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ps_version},
   {"svg_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.svg_version},
-  {"palette",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.Palette},
   {"main_pane",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.main_pane_pos},
   {"side_pane1",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.side_pane1_pos},
   {"side_pane2",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.side_pane2_pos},
@@ -221,6 +214,7 @@ static struct menu_config MenuConfigOthers[] = {
   {"arc_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.arc_tab},
   {"mark_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.mark_tab},
   {"text_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.text_tab},
+  {"parameter_tab",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.parameter_tab},
   {"math_input_mode",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.math_input_mode},
   {NULL},
 };
@@ -240,7 +234,6 @@ static struct menu_config MenuConfigExtView[] = {
 
 static struct menu_config *MenuConfigArrray[] = {
   MenuConfig,
-  MenuConfigDriver,
   MenuConfigScript,
   MenuConfigMisc,
   MenuConfigViewer,
@@ -308,7 +301,6 @@ add_color_ary_to_array(struct menu_config *cfg, struct narray *conf)
   struct narray *palette;
   int i, n;
   GdkRGBA *color;
-  unsigned int r, g, b, a;
   palette = &(Menulocal.custom_palette);
   n = arraynum(palette);
   if (n < 1) {
@@ -318,17 +310,18 @@ add_color_ary_to_array(struct menu_config *cfg, struct narray *conf)
   str = g_string_new(cfg->name);
   g_string_append_c(str, '=');
   for (i = 0; i < n; i++) {
-     color = arraynget(palette, i);
-     r = color->red * 0xff;
-     g = color->green * 0xff;
-     b = color->blue * 0xff;
-     a = color->alpha * 0xff;
-     g_string_append_printf(str, "%02x%02x%02x%02x%s", r, g, b, a, (i == n -1) ? "\n" : ",");
-   }
-   buf = g_string_free(str, FALSE);
-   if (buf) {
-     arrayadd(conf, &buf);
-   }
+    unsigned int r, g, b, a;
+    color = arraynget(palette, i);
+    r = color->red * 0xff;
+    g = color->green * 0xff;
+    b = color->blue * 0xff;
+    a = color->alpha * 0xff;
+    g_string_append_printf(str, "%02x%02x%02x%02x%s", r, g, b, a, (i == n -1) ? "\n" : ",");
+  }
+  buf = g_string_free(str, FALSE);
+  if (buf) {
+    arrayadd(conf, &buf);
+  }
 }
 
 static void
@@ -347,11 +340,12 @@ add_prm_str_to_array(struct menu_config *cfg, struct narray *conf)
 static void
 save_char_map_config(struct narray *conf)
 {
-  char *title, *data, *buf;
+  char *buf;
   struct character_map_list *pcur;
 
   pcur = Menulocal.char_map;
   while (pcur) {
+    char *title, *data;
     title = CHK_STR(pcur->title);
     data = CHK_STR(pcur->data);
 
@@ -363,34 +357,15 @@ save_char_map_config(struct narray *conf)
   }
 }
 
-static void
-save_ext_driver_config(struct narray *conf)
-{
-  char *buf, *driver, *ext, *option;
-  struct extprinter *pcur;
-
-  pcur = Menulocal.extprinterroot;
-  while (pcur) {
-    driver = CHK_STR(pcur->driver);
-    ext = CHK_STR(pcur->ext);
-    option= CHK_STR(pcur->option);
-
-    buf = g_strdup_printf("ext_driver=%s,%s,%s,%s", pcur->name, driver, ext, option);
-    if (buf) {
-      arrayadd(conf, &buf);
-    }
-    pcur = pcur->next;
-  }
-}
-
 static void
 save_script_config(struct narray *conf)
 {
-  char *buf, *script, *option, *description;
+  char *buf;
   struct script *scur;
 
   scur = Menulocal.scriptroot;
   while (scur) {
+    char *script, *option, *description;
     script = CHK_STR(scur->script);
     option = CHK_STR(scur->option);
     description = CHK_STR(scur->description);
@@ -440,9 +415,6 @@ menu_save_config_sub(struct menu_config *cfg, struct narray *conf)
     case MENU_CONFIG_TYPE_SCRIPT:
       save_script_config(conf);
       break;
-    case MENU_CONFIG_TYPE_DRIVER:
-      save_ext_driver_config(conf);
-      break;
     case MENU_CONFIG_TYPE_CHARMAP:
       save_char_map_config(conf);
       break;
@@ -477,10 +449,6 @@ menu_save_config(int type)
     menu_save_config_sub(MenuConfigOthers, &conf);
   }
 
-  if (type & SAVE_CONFIG_TYPE_EXTERNAL_DRIVER) {
-    menu_save_config_sub(MenuConfigDriver, &conf);
-  }
-
   if (type & SAVE_CONFIG_TYPE_ADDIN_SCRIPT) {
     menu_save_config_sub(MenuConfigScript, &conf);
   }
@@ -494,12 +462,6 @@ menu_save_config(int type)
 
   arrayinit(&conf, sizeof(char *));
 
-  if (type & SAVE_CONFIG_TYPE_EXTERNAL_DRIVER) {
-    if (Menulocal.extprinterroot == NULL) {
-      add_str_to_array(&conf, "ext_driver");
-    }
-  }
-
   if (type & SAVE_CONFIG_TYPE_ADDIN_SCRIPT) {
     if (Menulocal.scriptroot == NULL) {
       add_str_to_array(&conf, "script");
@@ -622,57 +584,6 @@ menu_config_set_char_map(char *s2, void *data)
   return 0;
 }
 
-static int
-menu_config_set_ext_driver(char *s2, void *data)
-{
-  char *f[4] = {NULL, NULL, NULL, NULL};
-  int len, i;
-  struct extprinter *pnew, *pcur, **pptr;
-
-  pptr = (struct extprinter **) data;
-  pcur = *pptr;
-
-  f[0] = getitok2(&s2, &len, ",");
-  f[1] = getitok2(&s2, &len, ",");
-
-  if (s2[1] == ',') {
-    f[2] = NULL;
-  } else {
-    f[2] = getitok2(&s2, &len, ",");
-  }
-
-  for (; (s2[0] != '\0') && (strchr(" \t,", s2[0])); s2++);
-
-  f[3] = getitok2(&s2, &len, "");
-
-  if (f[0] && f[1]) {
-    pnew = (struct extprinter *) g_malloc(sizeof(struct extprinter));
-    if (pnew == NULL) {
-      for (i = 0; i < 4; i++) {
-	g_free(f[i]);
-      }
-      return 1;
-    }
-    if (pcur == NULL) {
-      Menulocal.extprinterroot = pnew;
-    } else {
-      pcur->next = pnew;
-    }
-    *pptr = pnew;
-    pcur = pnew;
-    pcur->next = NULL;
-    pcur->name = f[0];
-    pcur->driver = f[1];
-    pcur->ext = f[2];
-    pcur->option = f[3];
-  } else {
-    for (i = 0; i < 4; i++) {
-      g_free(f[i]);
-    }
-  }
-  return 0;
-}
-
 static int
 menu_config_set_script(char *s2, void *data)
 {
@@ -728,7 +639,6 @@ mgtkloadconfig(void)
   int val;
   char *endptr;
   int len;
-  struct extprinter *pcur;
   struct script *scur;
   struct menu_config *cfg;
 
@@ -736,15 +646,8 @@ mgtkloadconfig(void)
   if (fp == NULL)
     return 0;
 
-  pcur = Menulocal.extprinterroot;
   scur = Menulocal.scriptroot;
 
-  if (nhash_get_ptr(MenuConfigHash, "ext_driver", (void *) &cfg) == 0) {
-    if (cfg) {
-      cfg->data = &pcur;
-    }
-  }
-
   if (nhash_get_ptr(MenuConfigHash, "script", (void *) &cfg) == 0) {
     if (cfg) {
       cfg->data = &scur;
@@ -786,7 +689,6 @@ mgtkloadconfig(void)
       case MENU_CONFIG_TYPE_COLOR:
       case MENU_CONFIG_TYPE_COLOR_ARY:
       case MENU_CONFIG_TYPE_SCRIPT:
-      case MENU_CONFIG_TYPE_DRIVER:
       case MENU_CONFIG_TYPE_WINDOW:
 	if (cfg->proc && cfg->proc(s2, cfg->data)) {
 	  g_free(tok);
@@ -826,10 +728,11 @@ menuadddrawrable(struct objlist *parent, struct narray *drawrable)
 static void
 free_script_list(struct script *script)
 {
-  struct script *scur, *sdel;
+  struct script *scur;
 
   scur = script;
   while (scur) {
+    struct script *sdel;
     sdel = scur;
     scur = scur->next;
     g_free(sdel->name);
@@ -882,7 +785,7 @@ init_layer(const char *obj)
 {
   struct layer *layer;
   void *ptr;
-  int r, w, h, lw, lh;
+  int r, w, h;
   if (Menulocal.pix == NULL) {
     return;
   }
@@ -900,6 +803,7 @@ init_layer(const char *obj)
     create_layer(layer, w, h);
     nhash_set_ptr(Menulocal.layers, obj, layer);
   } else {
+    int lw, lh;
     layer = ptr;
     lw = cairo_image_surface_get_width(layer->pix);
     lh = cairo_image_surface_get_height(layer->pix);
@@ -915,7 +819,6 @@ init_layer(const char *obj)
 static void
 menulocal_finalize(void)
 {
-  struct extprinter *pcur, *pdel;
   struct character_map_list *cmap, *cmap_tmp;
   int i, j;
   struct menu_config *cfg;
@@ -939,18 +842,6 @@ menulocal_finalize(void)
   }
   Menulocal.char_map = NULL;
 
-  pcur = Menulocal.extprinterroot;
-  while (pcur) {
-    pdel = pcur;
-    pcur = pcur->next;
-    g_free(pdel->name);
-    g_free(pdel->driver);
-    g_free(pdel->ext);
-    g_free(pdel->option);
-    g_free(pdel);
-  }
-  Menulocal.extprinterroot = NULL;
-
   free_script_list(Menulocal.scriptroot);
   Menulocal.scriptroot = NULL;
 
@@ -1024,6 +915,10 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     return 1;
   }
 
+#if GTK_SOURCE_CHECK_VERSION(4, 0, 0)
+  gtk_source_init();
+#endif
+
   layer = TRUE;
   if (_putobj(obj, "_layer", inst, &layer)) return 1;
 
@@ -1056,6 +951,7 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.exwin_use_external = TRUE;
   Menulocal.expand = 1;
   Menulocal.expanddir = g_strdup("./");
+  Menulocal.help_file = g_strdup("html/index.html");
   Menulocal.source_style_id = NULL;
   Menulocal.loadpath = SAVE_PATH_FULL;
   Menulocal.GRAobj = chkobject("gra");
@@ -1072,6 +968,7 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.file_tab = 0;
   Menulocal.axis_tab = 100;
   Menulocal.merge_tab = 101;
+  Menulocal.parameter_tab = 102;
   Menulocal.path_tab = 1;
   Menulocal.rectangle_tab = 2;
   Menulocal.arc_tab = 3;
@@ -1174,6 +1071,10 @@ menudone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
 
   menulocal_finalize();
 
+#if GTK_SOURCE_CHECK_VERSION(4, 0, 0)
+  gtk_source_finalize();
+#endif
+
   return 0;
 }
 
@@ -1200,7 +1101,10 @@ mgtkputstderr(const char *s)
 int
 mgtkputstdout(const char *s)
 {
-  return PutStdout(s);
+  int r;
+  r = PutStdout(s);
+  PutStdout("\n");
+  return r;
 }
 
 int
@@ -1243,7 +1147,10 @@ mgtkinputyn(const char *mes)
   char *ptr;
   int r;
 
-  ptr = g_locale_to_utf8(CHK_STR(mes), -1, NULL, NULL, NULL);
+  ptr = n_locale_to_utf8(CHK_STR(mes));
+  if (ptr == NULL) {
+    return 0;
+  }
   r = InputYN(ptr);
   g_free(ptr);
 
@@ -1361,16 +1268,12 @@ main_window_redraw(void)
 }
 
 void
-mx_redraw(struct objlist *obj, N_VALUE *inst, char **objects)
+mx_redraw(struct objlist *obj, N_VALUE *inst, char const **objects)
 {
   int n;
-  char *objs[OBJ_MAX];
+  char const *objs[OBJ_MAX];
   struct savedstdio save;
 
-  if (Menulocal.region) {
-    mx_clear(Menulocal.region, objects);
-  }
-
   if (Menulocal.redrawf) {
     n = Menulocal.redrawf_num;
   } else {
@@ -1389,6 +1292,7 @@ mx_redraw(struct objlist *obj, N_VALUE *inst, char **objects)
     objects = objs;
   }
 
+  mx_clear(NULL, objects);
   ignorestdio(&save);
   GRAredraw_layers(obj, inst, TRUE, n, objects);
   restorestdio(&save);
@@ -1397,25 +1301,6 @@ mx_redraw(struct objlist *obj, N_VALUE *inst, char **objects)
   main_window_redraw();
 }
 
-void
-mx_inslist(struct objlist *obj, N_VALUE *inst,
-	   struct objlist *aobj, N_VALUE *ainst, char *afield, int addn)
-{
-  int gc;
-
-  _getobj(obj, "_GC", inst, &gc);
-  GRAinslist(gc, aobj, ainst, chkobjectname(aobj), afield, addn);
-}
-
-void
-mx_dellist(struct objlist *obj, N_VALUE *inst, int deln)
-{
-  int gc;
-
-  _getobj(obj, "_GC", inst, &gc);
-  GRAdellist(gc, deln);
-}
-
 static int
 mxredraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -1456,15 +1341,16 @@ mxdpi(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 static void
 flush_layers(cairo_t *cr)
 {
-  int r, i, n;
+  int i, n;
   struct narray *array;
   struct layer *layer;
-  char *obj;
   void *ptr;
 
   array = &Menulocal.drawrable;
   n = arraynum(array);
   for (i = 0; i < n; i++) {
+    int r;
+    char *obj;
     obj = arraynget_str(array, i);
     r = nhash_get_ptr(Menulocal.layers, obj, &ptr);
     if (r) {
@@ -1498,14 +1384,13 @@ clear_region(cairo_t *cr, cairo_region_t *region)
 static int
 mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  cairo_surface_t *surface;
-  cairo_t *cr;
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
   }
 
   if (Menulocal.local->cairo) {
+    cairo_surface_t *surface;
     gra2cairo_draw_path(Menulocal.local);
 
     surface = cairo_get_target(Menulocal.local->cairo);
@@ -1515,6 +1400,7 @@ mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   }
 
   if (Menulocal.pix) {
+    cairo_t *cr;
     cr = cairo_create(Menulocal.pix);
     clear_region(cr, NULL);
     flush_layers(cr);
@@ -1540,21 +1426,20 @@ clear_layer(const char *obj, cairo_region_t *region)
 }
 
 void
-mx_clear(cairo_region_t *region, char **objects)
+mx_clear(cairo_region_t *region, char const **objects)
 {
-  int i, n;
-  struct narray *array;
-  char *obj;
-
   if (objects) {
     while(*objects) {
       clear_layer(*objects, region);
       objects++;
     }
   } else {
+    int i, n;
+    struct narray *array;
     array = &Menulocal.drawrable;
     n = arraynum(array);
     for (i = 0; i < n; i++) {
+      char *obj;
       obj = arraynget_str(array, i);
       clear_layer(obj, region);
     }
@@ -1676,12 +1561,12 @@ static int
 mx_print(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   int show_dialog, select_file, create_window = FALSE, lock;
-  GtkWidget *label;
 
   select_file = * (int *) argv[2];
   show_dialog = * (int *) argv[3];
 
   if (TopLevel == NULL) {
+    GtkWidget *label;
     create_window = TRUE;
     TopLevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_type_hint(GTK_WINDOW(TopLevel), GDK_WINDOW_TYPE_HINT_DIALOG);
@@ -1771,8 +1656,7 @@ mx_clear_info(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
 static int
 mx_get_accel_map(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-#if USE_GTK_BUILDER
-  char **actions, **accels;
+  char **actions;
   int i, j;
   GString *str;
 
@@ -1781,9 +1665,14 @@ mx_get_accel_map(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, ch
   }
   rval->str = NULL;
 
+  if (GtkApp == NULL) {
+    return 0;
+  }
+
   str = g_string_new("");
   actions = gtk_application_list_action_descriptions(GtkApp);
   for (i = 0; actions[i]; i++) {
+    char **accels;
     accels = gtk_application_get_accels_for_action(GtkApp, actions[i]);
     for (j = 0; accels[j]; j++) {
       g_string_append_printf(str, "%s %s\n", actions[i], accels[j]);
@@ -1791,36 +1680,6 @@ mx_get_accel_map(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, ch
     g_strfreev(accels);
   }
   g_strfreev(actions);
-#else
-  FILE *fp;
-  int fd;
-  char buf[1024], *ptr;
-  GString *str;
-
-  if (rval->str) {
-    g_free(rval->str);
-  }
-  rval->str = NULL;
-
-  fp = tmpfile();
-
-  if (fp == NULL) {
-    putstderr(g_strerror(errno));
-    return 1;
-  }
-
-  fd = fileno(fp);
-  gtk_accel_map_save_fd(fd);
-
-  rewind(fp);
-
-  str = g_string_new("");
-  while ((ptr = fgets(buf, sizeof(buf), fp))) {
-    buf[sizeof(buf) - 1] = '\0';
-    g_string_append(str, buf);
-  }
-  fclose(fp);
-#endif
 
   rval->str = g_string_free(str, FALSE);
 
@@ -2292,10 +2151,9 @@ static struct objtable gtkmenu[] = {
 void *
 addmenu(void)
 {
-  unsigned int i, j;
-  struct menu_config *cfg;
-
   if (MenuConfigHash == NULL) {
+    unsigned int i, j;
+    struct menu_config *cfg;
     MenuConfigHash = nhash_new();
     if (MenuConfigHash ==NULL)
       return NULL;
