@@ -52,15 +52,6 @@ enum paper_id {
   PAPER_ID_CUSTOM,
 };
 
-struct extprinter
-{
-  char *name;
-  char *driver;
-  char *ext;
-  char *option;
-  struct extprinter *next;
-};
-
 struct script
 {
   char *name;
@@ -70,15 +61,6 @@ struct script
   struct script *next;
 };
 
-struct prnprinter
-{
-  char *name;
-  char *driver;
-  char *option;
-  char *prn;
-  struct prnprinter *next;
-};
-
 struct character_map_list {
   char *title, *data;
   struct character_map_list *next;
@@ -97,23 +79,21 @@ struct menulocal
   int windpi, data_head_lines;
   int grid, show_grid;
   int modified;
-  cairo_region_t *region;
   int lock;
   struct gra2cairo_local *local;
   int antialias;
-  char *editor, *browser, *help_browser;
+  char *editor, *browser, *help_browser, *help_file;
   struct objlist *obj;
   N_VALUE *inst;
   struct objlist *GRAobj;
   int GRAoid;
   int GC;
   int PaperWidth, PaperHeight, PaperLandscape;
-  char *PaperName, *Palette;
+  char *PaperName;
   enum paper_id PaperId;
   int LeftMargin, TopMargin;
   int PaperZoom;
   struct narray drawrable;
-  struct extprinter *extprinterroot;
   struct script *scriptroot, *addin_list;
   int menuheight, menuwidth, menux, menuy;
   int exwindpi, exwinwidth, exwinheight, exwin_use_external;
@@ -126,7 +106,7 @@ struct menulocal
   double bg_r, bg_g, bg_b;
   int focus_frame_type, use_opacity, select_data;
   int side_pane1_pos, side_pane2_pos, side_pane3_pos, main_pane_pos;
-  int file_tab, axis_tab, merge_tab, path_tab, rectangle_tab, arc_tab, mark_tab, text_tab;
+  int file_tab, axis_tab, merge_tab, path_tab, rectangle_tab, arc_tab, mark_tab, text_tab, parameter_tab;
 #ifdef WINDOWS
   int emf_dpi;
 #endif
@@ -143,7 +123,6 @@ extern struct menulocal Menulocal;
 enum SAVE_CONFIG_TYPE {
   SAVE_CONFIG_TYPE_GEOMETRY        = 0x0001,
   SAVE_CONFIG_TYPE_VIEWER          = 0x0002,
-  SAVE_CONFIG_TYPE_EXTERNAL_DRIVER = 0x0004,
   SAVE_CONFIG_TYPE_ADDIN_SCRIPT    = 0x0008,
   SAVE_CONFIG_TYPE_MISC            = 0x0010,
   SAVE_CONFIG_TYPE_EXTERNAL_VIEWER = 0x0020,
@@ -155,15 +134,11 @@ enum SAVE_CONFIG_TYPE {
 #define SAVE_CONFIG_TYPE_X11MENU 	(SAVE_CONFIG_TYPE_GEOMETRY		\
 				  | SAVE_CONFIG_TYPE_VIEWER		\
 				  | SAVE_CONFIG_TYPE_EXTERNAL_VIEWER	\
-				  | SAVE_CONFIG_TYPE_EXTERNAL_DRIVER	\
 				  | SAVE_CONFIG_TYPE_ADDIN_SCRIPT	\
 				  | SAVE_CONFIG_TYPE_MISC)
 
-void mx_redraw(struct objlist *obj, N_VALUE *inst, char **objects);
-void mx_clear(cairo_region_t *region, char **objects);
-void mx_inslist(struct objlist *obj, N_VALUE *inst,
-		struct objlist *aobj, N_VALUE *ainst, char *afield, int addn);
-void mx_dellist(struct objlist *obj, N_VALUE *inst, int deln);
+void mx_redraw(struct objlist *obj, N_VALUE *inst, char const **objects);
+void mx_clear(cairo_region_t *region, char const **objects);
 void mgtkdisplaydialog(const char *str);
 void mgtkdisplaystatus(const char *str);
 int mgtkputstderr(const char *s);
