@@ -68,7 +68,6 @@ static void ToRalativePath(void);
 void
 OpenGRA(void)
 {
-  unsigned int j;
   int i, id, otherGC;
   char *device, *name, *name_str = "viewer";
   struct narray *drawrable;
@@ -113,6 +112,7 @@ OpenGRA(void)
   }
 
   if (arraynum(&(Menulocal.drawrable)) > 0) {
+    unsigned int j;
     drawrable = arraynew(sizeof(char *));
     for (j = 0; j < arraynum(&(Menulocal.drawrable)); j++) {
       arrayadd2(drawrable, *(char **)arraynget(&(Menulocal.drawrable), j));
@@ -214,9 +214,8 @@ ChangeGRA(void)
 void
 SetPageSettingsToGRA(void)
 {
-  int i, j, num, otherGC, id;
+  int i, otherGC;
   struct objlist *obj;
-  N_VALUE *inst;
   struct narray *drawrable;
 
   if ((obj = chkobject("gra")) == NULL)
@@ -229,6 +228,8 @@ SetPageSettingsToGRA(void)
   }
 
   if (i >= 0) {
+    int id, j, num;
+    N_VALUE *inst;
     id = i;
     inst = chkobjinst(obj, id);
 
@@ -257,9 +258,8 @@ SetPageSettingsToGRA(void)
 void
 GetPageSettingsFromGRA(void)
 {
-  int i, j, num, otherGC, id;
-  struct objlist *obj, *dobj;
-  N_VALUE *inst;
+  int i, otherGC;
+  struct objlist *obj;
   struct narray *drawrable;
 
   if ((obj = chkobject("gra")) == NULL)
@@ -272,6 +272,8 @@ GetPageSettingsFromGRA(void)
   }
 
   if (i >= 0) {
+    int id, num;
+    N_VALUE *inst;
     id = i;
     inst = chkobjinst(obj, id);
     CheckPage();
@@ -282,8 +284,10 @@ GetPageSettingsFromGRA(void)
     if (num == 0) {
       menuadddrawrable(chkobject("draw"), &(Menulocal.drawrable));
     } else {
-      char *name;
+      struct objlist *dobj;
+      int j;
       for (j = 0; j < num; j++) {
+        char *name;
 	name = arraynget_str(drawrable, j);
 	if (name == NULL) {
 	  continue;
@@ -340,13 +344,13 @@ static void
 AxisDel2(int id)
 {
   struct objlist *obj, *aobj;
-  int i, aid1, aid2;
+  int i, aid1;
   char *axis2;
-  N_VALUE *inst;
 
   obj = getobject("axisgrid");
   if (obj) {
     for (i = chkobjlastinst(obj); i >= 0; i--) {
+      N_VALUE *inst;
       inst = chkobjinst(obj, i);
       if (inst == NULL) {
 	continue;
@@ -370,6 +374,7 @@ AxisDel2(int id)
     return;
   }
   for (i = 0; i <= chkobjlastinst(obj); i++) {
+    int aid2;
     aid1 = get_new_axis_id(obj, &aobj, i, id, AXIS_X);
     aid2 = get_new_axis_id(obj, &aobj, i, id, AXIS_Y);
     if ((aid1 >= 0) && (aid2 >= 0)) {
@@ -397,7 +402,7 @@ AxisDel(int id)
   int i, lastinst, *id_array, n;
   char *group, *group2;
   char type;
-  N_VALUE *inst, *inst2;
+  N_VALUE *inst;
   char group3[20];
 
   obj = chkobject("axis");
@@ -426,6 +431,7 @@ AxisDel(int id)
 
   n = 0;
   for (i = lastinst; i >= 0; i--) {
+    N_VALUE *inst2;
     inst2 = chkobjinst(obj, i);
     _getobj(obj, "group", inst2, &group2);
     if (group2 && group2[0] == type) {
@@ -447,8 +453,8 @@ static void
 axis_move_each_obj(char *axis_str, int i, struct objlist *obj, int id1, int id2)
 {
   struct objlist *aobj;
-  int spc, aid;
-  char *axis, *axis2;
+  int spc;
+  char *axis;
   struct narray iarray;
   int anum;
 
@@ -461,6 +467,8 @@ axis_move_each_obj(char *axis_str, int i, struct objlist *obj, int id1, int id2)
 
   anum = arraynum(&iarray);
   if (anum > 0 && spc == OBJ_LIST_SPECIFIED_BY_ID) {
+    int aid;
+    char *axis2;
     aid = arraylast_int(&iarray);
     if (aid == id1) {
       aid = id2;
@@ -502,17 +510,17 @@ AxisNameToGroup(void)		/* this function may exist for compatibility with older v
   struct objlist *obj;
   char *name, *name2, *gp;
   struct narray group, iarray;
-  int a, j, anum;
+  int a, j;
   char *argv[2];
-  int *data;
-  N_VALUE *inst;
-
 
   if ((obj = (struct objlist *) chkobject("axis")) == NULL)
     return;
   num = chkobjlastinst(obj);
   arrayinit(&iarray, sizeof(int));
   for (id = 0; id <= num; id++) {
+    int anum;
+    int *data;
+    N_VALUE *inst;
     anum = arraynum(&iarray);
     data = arraydata(&iarray);
     for (j = 0; j < anum; j++)
@@ -685,15 +693,16 @@ field_obj_del(struct objlist *obj, int id, const char *field)
 {
   char *fit;
   struct objlist *fitobj;
-  int fitid, idnum, i;
   struct narray iarray;
 
   if ((getobj(obj, field, id, 0, NULL, &fit) >= 0) && (fit != NULL)) {
     arrayinit(&iarray, sizeof(int));
     if (getobjilist(fit, &fitobj, &iarray, FALSE, NULL) == 0) {
+      int idnum, i;
       idnum = arraynum(&iarray);
       arraysort_int(&iarray);
       for (i = idnum - 1; i >= 0; i--) {
+        int fitid;
 	fitid = arraynget_int(&iarray, i);
 	delobj(fitobj, fitid);
       }
@@ -719,24 +728,23 @@ FitCopy(struct objlist *obj, int did, int sid)
 {
   char *fit;
   struct objlist *fitobj;
-  int fitid;
   struct narray iarray;
   struct narray iarray2;
-  int id2, idnum, idnum2;
-  char *field[] = {"name", "equation", NULL};
   int fitoid;
-  N_VALUE *inst;
 
   if ((getobj(obj, "fit", sid, 0, NULL, &fit) >= 0) && (fit != NULL)) {
     arrayinit(&iarray, sizeof(int));
     if (getobjilist(fit, &fitobj, &iarray, FALSE, NULL) == 0) {
+      int idnum;
       idnum = arraynum(&iarray);
       if (idnum >= 1) {
+        int fitid, id2;
 	fitid = arraylast_int(&iarray);
 	if ((getobj(obj, "fit", did, 0, NULL, &fit) >= 0)
 	    && (fit != NULL)) {
 	  arrayinit(&iarray2, sizeof(int));
 	  if (getobjilist(fit, &fitobj, &iarray2, FALSE, NULL) == 0) {
+            int idnum2;
 	    idnum2 = arraynum(&iarray2);
 	    if (idnum2 >= 1) {
 	      id2 = arraylast_int(&iarray2);
@@ -745,9 +753,12 @@ FitCopy(struct objlist *obj, int did, int sid)
 	  } else
 	    id2 = newobj(fitobj);
 	  arraydel(&iarray2);
-	} else
+	} else {
 	  id2 = newobj(fitobj);
+        }
 	if (id2 >= 0) {
+          char *field[] = {"name", "equation", NULL};
+          N_VALUE *inst;
 	  copy_obj_field(fitobj, id2, fitid, field);
 	  inst = getobjinst(fitobj, id2);
 	  _getobj(fitobj, "oid", inst, &fitoid);
@@ -800,7 +811,7 @@ static void
 del_darray(struct objlist *data_obj)
 {
   char *array;
-  int i, j, n, id, last;
+  int i, j, n, last;
   struct array_prm ary;
   struct narray *id_array;
   struct objlist *darray_obj;
@@ -836,20 +847,34 @@ del_darray(struct objlist *data_obj)
 
   n = arraynum(id_array);
   for (i = 0; i < n; i++) {
+    int id;
     id = arraynget_int(id_array, n - 1 - i);
     delobj(darray_obj, id);
   }
   arrayfree(id_array);
 }
 
+static void
+delete_instances(struct objlist *obj)
+{
+  int instnum, i;
+  instnum = chkobjlastinst(obj);
+  if (instnum == -1) {
+    return;
+  }
+  for (i = instnum; i >= 0; i--) {
+    delobj(obj, i);
+  }
+}
+
 void
 DeleteDrawable(void)
 {
-  struct objlist *fileobj, *drawobj;
-  int i;
+  struct objlist *fileobj, *drawobj, *prmobj;
 
   fileobj = chkobject("data");
   if (fileobj) {
+    int i;
     for (i = 0; i <= chkobjlastinst(fileobj); i++) {
       FitDel(fileobj, i);
     }
@@ -860,6 +885,12 @@ DeleteDrawable(void)
   if (drawobj) {
     delchildobj(drawobj);
   }
+
+  prmobj = chkobject("parameter");
+  if (prmobj) {
+    delete_instances(prmobj);
+  }
+  set_toolbox_mode(TOOLBOX_MODE_TOOLBAR);
 }
 
 static void
@@ -1030,6 +1061,22 @@ save_data(struct objlist *ocur, int hFile, int storedata, int i, int *array_data
   nwrite(hFile, "\n", 1);
 }
 
+static void
+save_inst(int hFile, struct objlist *ocur)
+{
+  int i, instnum;
+  char *s;
+  instnum = chkobjlastinst(ocur);
+  if (instnum == -1) {
+    return;
+  }
+  for (i = 0; i <= instnum; i++) {
+    getobj(ocur, "save", i, 0, NULL, &s);
+    nwrite(hFile, s, strlen(s));
+    nwrite(hFile, "\n", 1);
+  }
+}
+
 static void
 SaveParent(int hFile, struct objlist *parent, int storedata,
 	   int storemerge)
@@ -1071,12 +1118,9 @@ int
 SaveDrawrable(char *name, int storedata, int storemerge)
 {
   int hFile;
-  struct objlist *sysobj, *drawobj, *graobj;
-  int id, len, error;
-  char *arg[2];
+  int error;
   struct narray sarray;
-  N_VALUE *inst;
-  char *ver, *sysname, *s, *opt, comment[COMMENT_BUF_SIZE];
+  char *ver, *sysname, *s, *opt;
 
   error = FALSE;
 
@@ -1085,6 +1129,10 @@ SaveDrawrable(char *name, int storedata, int storemerge)
   if (hFile < 0) {
     error = TRUE;
   } else {
+    int len;
+    struct objlist *sysobj, *drawobj, *prmobj;
+    N_VALUE *inst;
+    char comment[COMMENT_BUF_SIZE];
     sysobj = chkobject("system");
     inst = chkobjinst(sysobj, 0);
     _getobj(sysobj, "name", inst, &sysname);
@@ -1096,11 +1144,18 @@ SaveDrawrable(char *name, int storedata, int storemerge)
     if (nwrite(hFile, comment, len) != len)
       error = TRUE;
 
+    prmobj = chkobject("parameter");
+    if (prmobj) {
+      save_inst(hFile, prmobj);
+    }
     if ((drawobj = chkobject("draw")) != NULL) {
+      struct objlist *graobj;
       SaveParent(hFile, drawobj, storedata, storemerge);
       if ((graobj = chkobject("gra")) != NULL) {
+        int id;
 	id = ValidGRA();
 	if (id != -1) {
+          char *arg[2];
 	  arrayinit(&sarray, sizeof(char *));
 	  opt = "device";
 	  arrayadd(&sarray, &opt);
@@ -1177,7 +1232,6 @@ get_save_opt(int *sdata, int *smerge, int *path)
 int
 GraphSave(int overwrite)
 {
-  char mes[256];
   int path;
   int sdata, smerge;
   int ret;
@@ -1217,6 +1271,7 @@ GraphSave(int overwrite)
 
     ret = get_save_opt(&sdata, &smerge, &path);
     if (ret == IDOK) {
+      char mes[256];
       snprintf(mes, sizeof(mes), _("Saving `%.128s'."), file);
       SetStatusBar(mes);
       if(SaveDrawrable(file, sdata, smerge)) {
@@ -1256,12 +1311,12 @@ GraphSave(int overwrite)
 static void
 change_filename(char * (*func)(const char *))
 {
-  struct objlist *obj;
   int i;
   unsigned int j;
   char *file, *file2, *objname[] = {"data", "merge"};
 
   for (j = 0; j < sizeof(objname) / sizeof(*objname); j++) {
+    struct objlist *obj;
     obj = chkobject(objname[j]);
     if (obj == NULL)
       continue;
@@ -1322,9 +1377,9 @@ LoadNgpFile(char *file, int console, char *option)
 {
   struct objlist *sys;
   char *expanddir;
-  struct objlist *obj, *aobj;
+  struct objlist *obj;
   char *name;
-  int i, r, newid, allocnow = FALSE, tmp;
+  int r, newid, allocnow = FALSE, tmp;
   char *s;
   int len;
   char *argv[2];
@@ -1427,6 +1482,8 @@ LoadNgpFile(char *file, int console, char *option)
   _exeobj(obj, "security", inst, 1, argv);
 
   if (r == 0) {
+    struct objlist *aobj;
+    int i;
     if ((aobj = getobject("axis")) != NULL) {
       for (i = 0; i <= chkobjlastinst(aobj); i++)
 	exeobj(aobj, "tight", i, 0, NULL);
@@ -1515,13 +1572,14 @@ check_ref_axis(char *ref, const char *group)
 static int
 file_auto_scale(char *file_obj, struct objlist *aobj, int anum)
 {
-  int i, refother;
+  int i;
   double min, max, inc;
   char *group, *ref, *argv[2];
 
   argv[0] = (char *) file_obj;
   argv[1] = NULL;
   for (i = 0; i <= anum; i++) {
+    int refother;
     getobj(aobj, "min", i, 0, NULL, &min);
     getobj(aobj, "max", i, 0, NULL, &max);
     getobj(aobj, "inc", i, 0, NULL, &inc);
@@ -1597,9 +1655,8 @@ AdjustAxis(void)
 int
 CheckSave(void)
 {
-  int ret;
-
   if (get_graph_modified()) {
+    int ret;
     ret = message_box(TopLevel, _("This graph is modified.\nSave this graph?"),
 		      _("Confirm"), RESPONS_YESNOCANCEL);
     if (ret == IDYES) {
@@ -1703,17 +1760,18 @@ free_console(int allocnow)
 static char *
 get_plot_cb_str(struct objlist *obj, int id, int source)
 {
-  char *valstr, *s;
+  char *s;
   const char *str;
 
   str = get_plot_info_str(obj, id, source);
   if (str == NULL) {
-    return g_strdup("....................");
+    return g_strdup(FILL_STRING);
   }
 
   if (source == DATA_SOURCE_FILE) {
+    char *valstr;
     valstr = getbasename(str);
-    s = g_strdup_printf("%s", (valstr) ? valstr : "....................");
+    s = g_strdup_printf("%s", (valstr) ? valstr : FILL_STRING);
     if (valstr != NULL) {
       g_free(valstr);
     }
@@ -1724,6 +1782,22 @@ get_plot_cb_str(struct objlist *obj, int id, int source)
   return s;
 }
 
+char *
+MergeFileCB(struct objlist *obj, int id)
+{
+  char *file, *bname;
+
+  getobj(obj, "file", id, 0, NULL, &file);
+  if (file == NULL) {
+    return g_strdup(FILL_STRING);
+  }
+  bname = getbasename(file);
+  if (bname == NULL) {
+    return g_strdup(FILL_STRING);
+  }
+  return bname;
+}
+
 char *
 FileCB(struct objlist *obj, int id)
 {
@@ -1751,7 +1825,7 @@ SetFileHidden(void)
   struct objlist *fobj;
   int lastinst;
   struct narray farray, ifarray;
-  int i, a, r, num, inum, *array;
+  int i, a, r;
 
   fobj = chkobject("data");
   if (fobj == NULL) {
@@ -1774,6 +1848,7 @@ SetFileHidden(void)
   r = 0;
   SelectDialog(&DlgSelect, fobj, NULL, FileCB, &farray, &ifarray);
   if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
+    int num, inum, *array;
     a = TRUE;
     for (i = 0; i <= lastinst; i++) {
       putobj(fobj, "hidden", i, &a);
