@@ -161,14 +161,14 @@ PageDialogPage(GtkWidget *w, gpointer client_data)
 static void
 PageDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *table;
-  unsigned int j;
   struct PageDialog *d;
-  int i;
 
   d = (struct PageDialog *) data;
 
   if (makewidget) {
+    GtkWidget *w, *table;
+    unsigned int j;
+    int i;
     table = gtk_grid_new();
 
     i = 0;
@@ -264,7 +264,6 @@ static void
 SwitchDialogSetupItem(GtkWidget *w, struct SwitchDialog *d)
 {
   int j, num;
-  char **buf;
   GtkTreeIter iter;
 
   d->btn_lock = TRUE;
@@ -272,6 +271,7 @@ SwitchDialogSetupItem(GtkWidget *w, struct SwitchDialog *d)
   list_store_clear(d->drawlist);
   num = arraynum(&(d->idrawrable));
   for (j = 0; j < num; j++) {
+    char **buf;
     buf = (char **) arraynget(&(d->drawrable),
 			      arraynget_int(&(d->idrawrable), j));
     list_store_append(d->drawlist, &iter);
@@ -287,7 +287,7 @@ SwitchDialogAdd(GtkWidget *w, gpointer client_data)
   struct SwitchDialog *d;
   GtkTreeSelection *selected;
   GList *list, *ptr;
-  int duplicate, num, *data, i;
+  int num, *data, i;
 
   d = (struct SwitchDialog *) client_data;
 
@@ -298,7 +298,7 @@ SwitchDialogAdd(GtkWidget *w, gpointer client_data)
   num = arraynum(&(d->idrawrable));
 
   for (ptr = list; ptr; ptr = ptr->next) {
-    int *ary, a;
+    int *ary, a, duplicate;
 
     duplicate = FALSE;
     ary = gtk_tree_path_get_indices((GtkTreePath *)(ptr->data));
@@ -309,7 +309,7 @@ SwitchDialogAdd(GtkWidget *w, gpointer client_data)
 	break;
       }
     }
-    if ( !duplicate) {
+    if (!duplicate) {
       arrayadd(&(d->idrawrable), &a);
     }
   }
@@ -425,7 +425,7 @@ SwitchDialogUp(GtkWidget *w, gpointer client_data)
   GtkTreeSelection *selected;
   GList *list, *ptr;
   struct SwitchDialog *d;
-  int i, k, modified, *ary;
+  int k, modified, *ary;
   GtkTreePath *path;
 
   d = (struct SwitchDialog *) client_data;
@@ -438,6 +438,7 @@ SwitchDialogUp(GtkWidget *w, gpointer client_data)
 
   modified = FALSE;
   for (ptr = list; ptr; ptr = g_list_next(ptr)) {
+    int i;
     path = (GtkTreePath *) ptr->data;
     ary = gtk_tree_path_get_indices(path);
 
@@ -481,7 +482,7 @@ SwitchDialogDown(GtkWidget *w, gpointer client_data)
   GtkTreeSelection *selected;
   GList *list, *ptr;
   struct SwitchDialog *d;
-  int i, k, num, modified, *ary;
+  int k, num, modified, *ary;
   GtkTreePath *path;
 
   d = (struct SwitchDialog *) client_data;
@@ -496,6 +497,7 @@ SwitchDialogDown(GtkWidget *w, gpointer client_data)
 
   modified = FALSE;
   for (ptr = g_list_last(list); ptr; ptr = g_list_previous(ptr)) {
+    int i;
     path = (GtkTreePath *) ptr->data;
     ary = gtk_tree_path_get_indices(path);
 
@@ -624,12 +626,12 @@ SwitchDialogRemove(GtkWidget *w, gpointer client_data)
 static gboolean
 drawlist_sel_cb(GtkTreeSelection *sel, gpointer user_data)
 {
-  int n;
   struct SwitchDialog *d;
 
   d = (struct SwitchDialog *) user_data;
 
   if (! d->btn_lock) {
+    int n;
     n = gtk_tree_selection_count_selected_rows(sel);
     set_drawlist_btn_state(d, n);
   }
@@ -639,12 +641,12 @@ drawlist_sel_cb(GtkTreeSelection *sel, gpointer user_data)
 static gboolean
 objlist_sel_cb(GtkTreeSelection *sel, gpointer user_data)
 {
-  int n;
   struct SwitchDialog *d;
 
   d = (struct SwitchDialog *) user_data;
 
   if (! d->btn_lock) {
+    int n;
     n = gtk_tree_selection_count_selected_rows(sel);
     set_objlist_btn_state(d, n);
   }
@@ -654,12 +656,10 @@ objlist_sel_cb(GtkTreeSelection *sel, gpointer user_data)
 static void
 SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *hbox, *vbox, *vbox2, *label, *frame;
   GtkTreeIter iter;
   struct SwitchDialog *d;
   int num2, num1, j, k, *obj_check;
   char **buf;
-  GtkTreeSelection *sel;
   static n_list_store list[] = {
     {N_("Object"), G_TYPE_STRING, TRUE, FALSE, NULL},
   };
@@ -667,6 +667,8 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
   d = (struct SwitchDialog *) data;
 
   if (makewidget) {
+    GtkWidget *w, *hbox, *vbox, *vbox2, *label, *frame;
+    GtkTreeSelection *sel;
 
     hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 
@@ -808,17 +810,18 @@ static void
 SwitchDialogClose(GtkWidget *w, void *data)
 {
   struct SwitchDialog *d;
-  int j, num;
-  char **buf;
 
   d = (struct SwitchDialog *) data;
   if (d->ret == IDOK) {
+    int num;
     arraydel2(&(Menulocal.drawrable));
     num = arraynum(&(d->idrawrable));
     if (num == 0) {
       menuadddrawrable(chkobject("draw"), &(Menulocal.drawrable));
     } else {
+      int j;
       for (j = 0; j < num; j++) {
+        char **buf;
 	buf = (char **) arraynget(&(d->drawrable),
 				  arraynget_int(&(d->idrawrable), j));
 	if ((*buf) != NULL)
@@ -842,12 +845,12 @@ SwitchDialog(struct SwitchDialog *data)
 static void
 DirectoryDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *table;
   struct DirectoryDialog *d;
   char *cwd;
 
   d = (struct DirectoryDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *table;
     table = gtk_grid_new();
 
     w = gtk_file_chooser_button_new(_("directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
@@ -912,12 +915,12 @@ DirectoryDialog(struct DirectoryDialog *data)
 static void
 LoadDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *vbox;
   struct LoadDialog *d;
-  int j;
 
   d = (struct LoadDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *vbox;
+    int j;
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
 
     w = gtk_check_button_new_with_mnemonic(_("_Expand included file"));
@@ -975,12 +978,12 @@ LoadDialog(struct LoadDialog *data)
 static void
 SaveDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *vbox;
-  int j;
   struct SaveDialog *d;
 
   d = (struct SaveDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *vbox;
+    int j;
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
 
     w = combo_box_create();
@@ -1050,13 +1053,13 @@ CmGraphNewMenu(void *w, gpointer client_data)
   sel = GPOINTER_TO_INT(client_data);
   switch (sel) {
   case MenuIdGraphNewFrame:
-    CmAxisNewFrame(NULL, client_data);
+    CmAxisNewFrame(FALSE);
     break;
   case MenuIdGraphNewSection:
-    CmAxisNewSection(NULL, client_data);
+    CmAxisNewSection(FALSE);
     break;
   case MenuIdGraphNewCross:
-    CmAxisNewCross(NULL, client_data);
+    CmAxisNewCross(FALSE);
     break;
   case MenuIdGraphAllClear:
   default:
@@ -1151,7 +1154,7 @@ CmGraphShell(void *w, gpointer client_data)
 {
   struct objlist *obj, *robj, *shell;
   N_VALUE *inst;
-  int idn, allocnow, n;
+  int idn;
 
   if (Menulock || Globallock)
     return;
@@ -1164,6 +1167,7 @@ CmGraphShell(void *w, gpointer client_data)
   idn = getobjtblpos(obj, "_evloop", &robj);
   registerevloop(chkobjectname(obj), "_evloop", robj, idn, inst, NULL);
   if ((shell = chkobject("shell")) != NULL) {
+    int allocnow, n;
     n = chkobjlastinst(shell);
     if (n < 0) {
       newobj(shell);
@@ -1253,45 +1257,42 @@ CmHelpAbout(void *w, gpointer client_data)
 			NULL);
 }
 
-static char *
-check_help_file(void)
+void
+CmHelpDemo(void *w, gpointer client_data)
 {
-  char *file, *ptr, *tmp;
-  const char *locale;
+  struct objlist *obj;
+  char *demo_file, *data_dir;
 
-  locale = n_getlocale();
-  if (locale == NULL) {
-    goto End;
+  if (Menulock || Globallock)
+    return;
+
+  obj = chkobject("system");
+  if (obj == NULL) {
+    return;
   }
 
-  tmp = g_strdup(locale);
-  if (tmp == NULL){
-    goto End;
+  getobj(obj, "data_dir", 0, 0, NULL, &data_dir);
+  if (data_dir == NULL) {
+    return;
   }
 
-  ptr = strchr(tmp, '_');
-  if (ptr == NULL) {
-    g_free(tmp);
-    goto End;
+  if (!CheckSave()) {
+    return;
   }
 
-  *ptr = '\0';
-  file = g_strdup_printf("%s/html/%s/%s", DOCDIR, tmp, HELP_FILE);
-  g_free(tmp);
-
-  if (naccess(file, R_OK) == 0) {
-    return file;
+  demo_file = g_strdup_printf("%s/demo/demo.ngp", data_dir);
+  if (demo_file == NULL) {
+    return;
   }
-  g_free(file);
 
- End:
-  return g_strdup_printf("%s/html/ja/%s", DOCDIR, HELP_FILE); /* default language of the help file is Japanese. */
+  LoadNgpFile(demo_file, Menulocal.scriptconsole, "-f");
+  g_free(demo_file);
 }
 
 void
 CmHelpHelp(void *w, gpointer client_data)
 {
-  char *cmd, *file;
+  char *cmd;
 
   if (Menulock || Globallock)
     return;
@@ -1299,10 +1300,7 @@ CmHelpHelp(void *w, gpointer client_data)
   if (Menulocal.help_browser == NULL)
     return;
 
-  file = check_help_file();
-
-  cmd = g_strdup_printf("%s \"%s\"", Menulocal.help_browser, file);
-  g_free(file);
+  cmd = g_strdup_printf("%s \"%s/%s\"", Menulocal.help_browser, DOCDIR, Menulocal.help_file);
 
   system_bg(cmd);
 
