@@ -104,9 +104,12 @@ ndialog_run(GtkWidget *dlg)
 int
 DialogExecute(GtkWidget *parent, void *dialog)
 {
-  GtkWidget *dlg, *win_ptr, *btn;
+  GtkWidget *dlg, *win_ptr;
   struct DialogType *data;
   gint res_id, lockstate;
+#if OSX
+  int menulock;
+#endif
 
   lockstate = DnDLock;
   DnDLock = TRUE;
@@ -127,7 +130,7 @@ DialogExecute(GtkWidget *parent, void *dialog)
   if (data->widget == NULL) {
     dlg = gtk_dialog_new_with_buttons(_(data->resource),
 				      GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				      GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -149,6 +152,7 @@ DialogExecute(GtkWidget *parent, void *dialog)
     gtk_dialog_add_button(GTK_DIALOG(dlg), data->ok_button, GTK_RESPONSE_OK);
 
     if (! data->show_cancel) {
+      GtkWidget *btn;
       btn = gtk_dialog_get_widget_for_response(GTK_DIALOG(dlg), GTK_RESPONSE_CANCEL);
       gtk_widget_hide(btn);
     }
@@ -164,6 +168,10 @@ DialogExecute(GtkWidget *parent, void *dialog)
   data->widget = dlg;
   data->ret = IDLOOP;
 
+#if OSX
+  menulock = Menulock;
+  Menulock = TRUE;
+#endif
   gtk_widget_show(dlg);
   win_ptr = get_current_window();
   set_current_window(dlg);
@@ -190,6 +198,9 @@ DialogExecute(GtkWidget *parent, void *dialog)
       data->CloseWindow(dlg, data);
     }
   }
+#if OSX
+  Menulock = menulock;
+#endif
 
   //  gtk_widget_destroy(dlg);
   //  data->widget = NULL;
@@ -347,7 +358,7 @@ DialogInput(GtkWidget * parent, const char *title, const char *mes, const char *
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -415,7 +426,7 @@ DialogRadio(GtkWidget *parent, const char *title, const char *caption, struct na
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -551,7 +562,7 @@ DialogCombo(GtkWidget *parent, const char *title, const char *caption, struct na
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -617,7 +628,6 @@ DialogComboEntry(GtkWidget *parent, const char *title, const char *caption, stru
   int data;
   gint res_id;
   char **d;
-  const char *s;
   int i, anum;
 
   d = arraydata(array);
@@ -627,7 +637,7 @@ DialogComboEntry(GtkWidget *parent, const char *title, const char *caption, stru
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -664,6 +674,7 @@ DialogComboEntry(GtkWidget *parent, const char *title, const char *caption, stru
   res_id = ndialog_run(dlg);
 
   if (res_id > 0 || res_id == GTK_RESPONSE_OK) {
+    const char *s;
     s = combo_box_entry_get_text(combo);
     if (s) {
       *r = g_strdup(s);
@@ -697,7 +708,7 @@ DialogSpinEntry(GtkWidget *parent, const char *title, const char *caption, doubl
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -776,7 +787,7 @@ DialogCheck(GtkWidget *parent, const char *title, const char *caption, struct na
 
   dlg = gtk_dialog_new_with_buttons(title,
 				    GTK_WINDOW(parent),
-#if GTK_CHECK_VERSION(3, 12, 0) && USE_HEADER_BAR
+#if USE_HEADER_BAR
 				    GTK_DIALOG_USE_HEADER_BAR |
 #endif
 				    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
