@@ -79,10 +79,6 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
     d->fonts = w;
     gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
 
-    w = gtk_check_button_new_with_mnemonic(_("_External Driver"));
-    d->external_driver = w;
-    gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
-
     w = gtk_check_button_new_with_mnemonic(_("_Add-in Script"));
     d->addin_script = w;
     gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
@@ -94,7 +90,6 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
   }
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->viewer), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->external_viewer), FALSE);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->external_driver), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->addin_script), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->misc), FALSE);
 }
@@ -128,7 +123,6 @@ DefaultDialogClose(GtkWidget *win, void *data)
     enum SAVE_CONFIG_TYPE type;
   } btns[] = {
     {NULL, SAVE_CONFIG_TYPE_VIEWER},
-    {NULL, SAVE_CONFIG_TYPE_EXTERNAL_DRIVER},
     {NULL, SAVE_CONFIG_TYPE_ADDIN_SCRIPT},
     {NULL, SAVE_CONFIG_TYPE_MISC},
     {NULL, SAVE_CONFIG_TYPE_EXTERNAL_VIEWER},
@@ -144,11 +138,10 @@ DefaultDialogClose(GtkWidget *win, void *data)
   d->ret = IDLOOP;
 
   btns[0].btn = d->viewer;
-  btns[1].btn = d->external_driver;
-  btns[2].btn = d->addin_script;
-  btns[3].btn = d->misc;
-  btns[4].btn = d->external_viewer;
-  btns[5].btn = d->fonts;
+  btns[1].btn = d->addin_script;
+  btns[2].btn = d->misc;
+  btns[3].btn = d->external_viewer;
+  btns[4].btn = d->fonts;
 
   type = 0;
 
@@ -232,7 +225,6 @@ static void
 SetScriptDialogSetupItem(GtkWidget *w, struct SetScriptDialog *d)
 {
   struct script *addin;
-  char *title;
 
   combo_box_clear(d->addins);
   combo_box_append_text(d->addins, "Custom");
@@ -241,6 +233,7 @@ SetScriptDialogSetupItem(GtkWidget *w, struct SetScriptDialog *d)
   } else {
     set_widget_sensitivity_with_label(d->addins, TRUE);
     for (addin = Menulocal.addin_list; addin; addin = addin->next) {
+      char *title;
       title = g_strdup(addin->name);
       if (title) {
 	remove_char(title, '_');
@@ -277,12 +270,12 @@ SetScriptDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *even
 static void
 SetScriptDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *table;
   struct SetScriptDialog *d;
-  int i;
 
   d = (struct SetScriptDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *table;
+    int i;
     table = gtk_grid_new();
 
     i = 0;
@@ -489,203 +482,6 @@ PrefScriptDialog(struct PrefScriptDialog *data)
   data->CloseWindow = PrefScriptDialogClose;
 }
 
-
-static void
-SetDriverDialogSetupItem(GtkWidget *w, struct SetDriverDialog *d)
-{
-  gtk_entry_set_text(GTK_ENTRY(d->name), CHK_STR(d->Driver->name));
-  gtk_entry_set_text(GTK_ENTRY(d->driver), CHK_STR(d->Driver->driver));
-  gtk_entry_set_text(GTK_ENTRY(d->option), CHK_STR(d->Driver->option));
-  gtk_entry_set_text(GTK_ENTRY(d->ext), CHK_STR(d->Driver->ext));
-}
-
-static void
-SetDriverDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
-{
-  char *file;
-  struct SetDriverDialog *d;
-
-  d = (struct SetDriverDialog *) user_data;
-  if (nGetOpenFileName(d->widget, _("External Driver"), NULL, NULL,
-		       NULL, &file, TRUE, FALSE) == IDOK) {
-    entry_set_filename(GTK_WIDGET(w), file);
-  }
-  g_free(file);
-}
-
-static void
-SetDriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
-{
-  GtkWidget *w, *table;
-  struct SetDriverDialog *d;
-  int i;
-
-  d = (struct SetDriverDialog *) data;
-  if (makewidget) {
-    table = gtk_grid_new();
-
-    i = 0;
-    w = create_text_entry(FALSE, TRUE);
-    add_widget_to_table(table, w, _("_Name:"), TRUE, i++);
-    d->name = w;
-
-    w = create_file_entry_with_cb(G_CALLBACK(SetDriverDialogBrowse), d);
-    add_widget_to_table(table, w, _("_Driver:"), TRUE, i++);
-    d->driver = w;
-
-    w = create_text_entry(FALSE, TRUE);
-    add_widget_to_table(table, w, _("_Option:"), TRUE, i++);
-    d->option = w;
-
-    w = create_text_entry(FALSE, TRUE);
-    add_widget_to_table(table, w, _("_Extension:"), TRUE, i++);
-    d->ext = w;
-
-    gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
-  }
-  SetDriverDialogSetupItem(wi, d);
-}
-
-static void
-SetDriverDialogClose(GtkWidget *w, void *data)
-{
-  const char *buf;
-  char *buf2;
-  int ret;
-  struct SetDriverDialog *d;
-
-  d = (struct SetDriverDialog *) data;
-
-  if (d->ret != IDOK)
-    return;
-
-  ret = d->ret;
-  d->ret = IDLOOP;
-
-  buf = gtk_entry_get_text(GTK_ENTRY(d->name));
-  if (strlen(buf) == 0) {
-    message_box(d->widget, _("Please specify driver name."), NULL, RESPONS_OK);
-    return;
-  }
-
-  buf2 = g_strdup(buf);
-  if (buf2) {
-    g_free(d->Driver->name);
-    d->Driver->name = buf2;
-  }
-
-  buf2 = entry_get_filename(d->driver);
-  if (buf2) {
-    g_free(d->Driver->driver);
-    d->Driver->driver = buf2;
-  }
-
-  buf = gtk_entry_get_text(GTK_ENTRY(d->ext));
-  buf2 = g_strdup(buf);
-  if (buf2) {
-    g_free(d->Driver->ext);
-    d->Driver->ext = buf2;
-  }
-
-  buf = gtk_entry_get_text(GTK_ENTRY(d->option));
-  buf2 = g_strdup(buf);
-  if (buf2) {
-    g_free(d->Driver->option);
-    d->Driver->option = buf2;
-  }
-
-  d->ret = ret;
-}
-
-void
-SetDriverDialog(struct SetDriverDialog *data, struct extprinter *prn)
-{
-  data->SetupWindow = SetDriverDialogSetup;
-  data->CloseWindow = SetDriverDialogClose;
-  data->Driver = prn;
-}
-
-static void
-PrefDriverDialogSetupItem(struct PrefDriverDialog *d)
-{
-  struct extprinter *fcur;
-  GtkTreeIter iter;
-
-  list_store_clear(d->list);
-  fcur = Menulocal.extprinterroot;
-  while (fcur) {
-    list_store_append(d->list, &iter);
-    list_store_set_string(d->list, &iter, 0, fcur->name);
-    fcur = fcur->next;
-  }
-}
-
-static void
-extprinter_free(struct extprinter *fdel)
-{
-  g_free(fdel->name);
-  g_free(fdel->driver);
-  g_free(fdel->option);
-  g_free(fdel->ext);
-  g_free(fdel);
-}
-
-static void
-extprinter_init(struct extprinter *fnew)
-{
-  fnew->next = NULL;
-  fnew->name = NULL;
-  fnew->driver = NULL;
-  fnew->option = NULL;
-  fnew->ext = NULL;
-}
-
-#define LIST_TYPE   extprinter
-#define LIST_ROOT   Menulocal.extprinterroot
-#define SET_DIALOG  DlgSetDriver
-#define LIST_FREE   extprinter_free
-#define LIST_INIT   extprinter_init
-#define CREATE_NAME(a, c) a ## Driver ## c
-#include "x11opt_proto.h"
-
-static void
-PrefDriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
-{
-  struct PrefDriverDialog *d;
-  n_list_store list[] = {
-    {N_("Driver"), G_TYPE_STRING, TRUE, FALSE, NULL},
-  };
-
-  d = (struct PrefDriverDialog *) data;
-  if (makewidget) {
-    gtk_dialog_add_button(GTK_DIALOG(wi), _("_Save"), IDSAVE);
-    PrefDriverDialogCreateWidgets(d, NULL, sizeof(list) / sizeof(*list), list);
-    gtk_window_set_default_size(GTK_WINDOW(wi), 400, 300);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
-  }
-  PrefDriverDialogSetupItem(d);
-}
-
-static void
-PrefDriverDialogClose(GtkWidget *w, void *data)
-{
-  struct PrefDriverDialog *d;
-
-  d = (struct PrefDriverDialog *) data;
-
-  if (d->ret == IDSAVE) {
-    save_config(SAVE_CONFIG_TYPE_EXTERNAL_DRIVER);
-  }
-}
-
-void
-PrefDriverDialog(struct PrefDriverDialog *data)
-{
-  data->SetupWindow = PrefDriverDialogSetup;
-  data->CloseWindow = PrefDriverDialogClose;
-}
-
 static void
 FontSettingDialogSetupItem(GtkWidget *w, struct FontSettingDialog *d)
 {
@@ -767,12 +563,12 @@ FontSettingDialogAddAlternative(GtkWidget *w, gpointer client_data)
 
   dialog = gtk_font_chooser_dialog_new(_("Alternative font"), NULL);
   if (ndialog_run(dialog) != GTK_RESPONSE_CANCEL) {
-    const gchar *font_name;
     PangoFontFamily *family;
     GtkTreeIter iter;
 
     family = gtk_font_chooser_get_font_family(GTK_FONT_CHOOSER(dialog));
     if (family) {
+      const gchar *font_name;
       font_name = pango_font_family_get_name(family);
       list_store_append(d->list, &iter);
       list_store_set_string(d->list, &iter, 0, font_name);
@@ -856,13 +652,13 @@ FontSettingDialogUpAlternative(GtkWidget *wi, gpointer data)
 static void
 FontSettingDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *hbox, *vbox, *table, *frame, *swin;
   struct FontSettingDialog *d;
-  GtkTreeSelection *sel;
 
   d = (struct FontSettingDialog *) data;
 
   if (makewidget) {
+    GtkWidget *w, *hbox, *vbox, *table, *frame, *swin;
+    GtkTreeSelection *sel;
     n_list_store list[] = {
       {N_("Font name"),  G_TYPE_STRING, TRUE, FALSE, NULL},
     };
@@ -971,7 +767,7 @@ FontSettingDialogClose(GtkWidget *wi, void *data)
 {
   struct FontSettingDialog *d;
   gchar *alias, *family, *font;
-  const gchar *font_name;
+  gchar *font_name;
   struct fontmap *fmap;
   GString *alt;
   GtkTreeIter iter;
@@ -1000,7 +796,12 @@ FontSettingDialogClose(GtkWidget *wi, void *data)
   }
 
   font_name = gtk_font_chooser_get_font(GTK_FONT_CHOOSER(d->font_b));
+  if (font_name == NULL) {
+    g_free(alias);
+    return;
+  }
   family = get_font_family(font_name);
+  g_free(font_name);
   if (family == NULL) {
     g_free(alias);
     return;
@@ -1126,7 +927,6 @@ PrefFontDialogAdd(GtkWidget *w, gpointer client_data)
 static void
 PrefFontDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *vbox;
   struct PrefFontDialog *d;
   n_list_store list[] = {
     {N_("alias"), G_TYPE_STRING, TRUE, FALSE, NULL},
@@ -1136,6 +936,7 @@ PrefFontDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct PrefFontDialog *) data;
   if (makewidget) {
+    GtkWidget *vbox;
     gtk_dialog_add_button(GTK_DIALOG(wi), _("_Save"), IDSAVE);
 
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
@@ -1226,9 +1027,9 @@ static void
 set_file_in_entry(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
   char *file;
-  struct SetDriverDialog *d;
+  struct MiscDialog *d;
 
-  d = (struct SetDriverDialog *) user_data;
+  d = (struct MiscDialog *) user_data;
   if (nGetOpenFileName(d->widget, _("Select program"), NULL, NULL,
 		       NULL, &file, TRUE, FALSE) == IDOK) {
     entry_set_filename(GTK_WIDGET(w), file);
@@ -1327,12 +1128,12 @@ edit_custom_palette(GtkWidget *w, gpointer data)
 static void
 MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *hbox2, *vbox2, *frame, *table;
   struct MiscDialog *d;
-  int i, j;
 
   d = (struct MiscDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *hbox2, *vbox2, *frame, *table;
+    int i, j;
     arrayinit(&(d->tmp_palette), sizeof(GdkRGBA));
     gtk_dialog_add_button(GTK_DIALOG(wi), _("_Save"), IDSAVE);
 
@@ -1489,19 +1290,20 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 static int
 set_font(char **cfg, GtkWidget *btn)
 {
-  const char *buf;
+  char *buf;
 
   buf = gtk_font_chooser_get_font(GTK_FONT_CHOOSER(btn));
   if (buf && *cfg) {
     if (strcmp(*cfg, buf)) {
       g_free(*cfg);
     } else {
+      g_free(buf);
       buf = NULL;
     }
   }
 
   if (buf) {
-    *cfg = g_strdup(buf);
+    *cfg = buf;
     return 1;
   }
 
@@ -1658,12 +1460,12 @@ use_external_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 static void
 ExViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *table;
   struct ExViewerDialog *d;
-  int i;
 
   d = (struct ExViewerDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *table;
+    int i;
     gtk_dialog_add_button(GTK_DIALOG(wi), _("_Save"), IDSAVE);
 
     table = gtk_grid_new();
@@ -1766,12 +1568,12 @@ load_file_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 static void
 ViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *table;
   struct ViewerDialog *d;
-  int i, j;
 
   d = (struct ViewerDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *table;
+    int i, j;
     gtk_dialog_add_button(GTK_DIALOG(wi), _("_Save"), IDSAVE);
 
     table = gtk_grid_new();
@@ -1974,17 +1776,6 @@ CmOptionPrefFont(void *w, gpointer client_data)
   DialogExecute(TopLevel, &DlgPrefFont);
 }
 
-#if 0
-static void
-CmOptionPrefDriver(void)
-{
-  if (Menulock || Globallock)
-    return;
-  PrefDriverDialog(&DlgPrefDriver);
-  DialogExecute(TopLevel, &DlgPrefDriver);
-}
-#endif
-
 void
 CmOptionScript(void *w, gpointer client_datavoid)
 {
