@@ -69,167 +69,6 @@ struct print_obj {
 
 static GtkPrintSettings *PrintSettings = NULL;
 
-static void
-DriverDialogSelectCB(GtkWidget *wi, gpointer client_data)
-{
-  int a, i, l, n;
-  struct extprinter *pcur;
-  struct DriverDialog *d;
-  char *ptr, ngp_ext[] = ".ngp";
-
-  d = (struct DriverDialog *) client_data;
-
-  a = combo_box_get_active(d->driver);
-
-  if (a < 0)
-    return;
-
-  pcur = Menulocal.extprinterroot;
-  i = 0;
-  while (pcur) {
-    if (i != a) {
-      pcur = pcur->next;
-      i++;
-      continue;
-    }
-
-    if (pcur->ext && pcur->ext[0] != '\0') {
-      if (NgraphApp.FileName) {
-	l = strlen(NgraphApp.FileName);
-	n = sizeof(ngp_ext) - 1;
-	if (l > n && strcasecmp(NgraphApp.FileName + l - n, ngp_ext) == 0) {
-	  l -= n;
-	}
-	ptr = g_strdup_printf("%.*s.%s", l, NgraphApp.FileName, CHK_STR(pcur->ext));
-	gtk_entry_set_text(GTK_ENTRY(d->file), CHK_STR(ptr));
-	g_free(ptr);
-      }
-      d->ext = pcur->ext;
-    } else {
-      gtk_entry_set_text(GTK_ENTRY(d->file), "");
-      d->ext = NULL;
-    }
-
-    gtk_entry_set_text(GTK_ENTRY(d->option), CHK_STR(pcur->option));
-    break;
-  }
-}
-
-static void
-DriverDialogBrowseCB(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
-{
-  char *file;
-  const char *str;
-  struct DriverDialog *d;
-
-  d = (struct DriverDialog *) user_data;
-
-  str = gtk_entry_get_text(w);
-
-  if (nGetSaveFileName(d->widget, _("External Driver Output"), d->ext, NULL,
-		       str, &file, TRUE, Menulocal.changedirectory) == IDOK) {
-    gtk_entry_set_text(w, file);
-  }
-  g_free(file);
-}
-
-static void
-DriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
-{
-  GtkWidget *w, *table;
-  struct DriverDialog *d;
-  struct extprinter *pcur;
-  int i, j;
-
-  d = (struct DriverDialog *) data;
-  if (makewidget) {
-
-    table = gtk_grid_new();
-
-    i = 0;
-    w = combo_box_create();
-    d->driver = w;
-    add_widget_to_table(table, w, _("_Driver:"), FALSE, i++);
-    g_signal_connect(d->driver, "changed", G_CALLBACK(DriverDialogSelectCB), d);
-
-    w = create_text_entry(FALSE, TRUE);
-    d->option = w;
-    add_widget_to_table(table, w, _("_Option:"), TRUE, i++);
-
-    w = create_file_entry_with_cb(G_CALLBACK(DriverDialogBrowseCB), d);
-    d->file = w;
-    add_widget_to_table(table, w, _("_File:"), TRUE, i++);
-
-    gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
-  }
-
-  combo_box_clear(d->driver);
-
-  pcur = Menulocal.extprinterroot;
-  while (pcur != NULL) {
-    combo_box_append_text(d->driver, pcur->name);
-    pcur = pcur->next;
-    j++;
-  }
-  combo_box_set_active(d->driver, 0);
-}
-
-static void
-DriverDialogClose(GtkWidget *w, void *data)
-{
-  int a, i;
-  struct extprinter *pcur;
-  struct DriverDialog *d;
-  const char *s, *file;
-  char *driver, *option;
-
-  d = (struct DriverDialog *) data;
-
-  if (d->ret == IDCANCEL)
-    return;
-
-  a = combo_box_get_active(d->driver);
-  pcur = Menulocal.extprinterroot;
-  i = 0;
-  while (pcur != NULL) {
-    if (i == a && pcur->driver) {
-      driver = g_strdup(pcur->driver);
-      if (driver) {
-	putobj(d->Obj, "driver", d->Id, driver);
-      }
-      break;
-    }
-    pcur = pcur->next;
-    i++;
-  }
-
-  s = gtk_entry_get_text(GTK_ENTRY(d->option));
-  file = gtk_entry_get_text(GTK_ENTRY(d->file));
-
-  option = NULL;
-
-  if (s || file) {
-    option = g_strdup_printf("%s%s%s%s",
-		  (file) ? "-o '" : "",
-		  CHK_STR(file),
-		  (file) ? "' " : "",
-		  CHK_STR(s));
-  }
-
-  putobj(d->Obj, "option", d->Id, option);
-}
-
-void
-DriverDialog(struct DriverDialog *data, struct objlist *obj, int id)
-{
-  data->SetupWindow = DriverDialogSetup;
-  data->CloseWindow = DriverDialogClose;
-  data->Obj = obj;
-  data->Id = id;
-  data->ext = NULL;
-}
-
 static void
 OutputDataDialogSetupItem(GtkWidget *w, struct OutputDataDialog *d)
 {
@@ -239,11 +78,11 @@ OutputDataDialogSetupItem(GtkWidget *w, struct OutputDataDialog *d)
 static void
 OutputDataDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *hbox;
   struct OutputDataDialog *d;
 
   d = (struct OutputDataDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *hbox;
     w = create_spin_entry(0, 200, 1, FALSE, TRUE);
     d->div_entry = w;
     hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
@@ -276,7 +115,7 @@ OutputDataDialog(struct OutputDataDialog *data, int div)
 }
 
 static void
-OutputImageDialogSetupItem(GtkWidget *w, struct OutputImageDialog *d)
+OutputImageDialogSetupItem(struct OutputImageDialog *d)
 {
   int i;
   GtkWidget *vlabel;
@@ -408,7 +247,7 @@ OutputImageDialogSetup(GtkWidget *wi, void *data, int makewidget)
   }
 
   gtk_window_set_title(GTK_WINDOW(wi), _(title));
-  OutputImageDialogSetupItem(w, d);
+  OutputImageDialogSetupItem(d);
 }
 
 static void
@@ -541,7 +380,6 @@ static void
 init_graobj(struct objlist *graobj, int id, char *dev_name, int dev_oid)
 {
   struct narray *drawrable;
-  unsigned int i, n;
   char *device;
 
   putobj(graobj, "paper_width", id, &(Menulocal.PaperWidth));
@@ -550,6 +388,7 @@ init_graobj(struct objlist *graobj, int id, char *dev_name, int dev_oid)
   putobj(graobj, "top_margin", id, &(Menulocal.TopMargin));
   putobj(graobj, "zoom", id, &(Menulocal.PaperZoom));
   if (arraynum(&(Menulocal.drawrable)) > 0) {
+    unsigned int i, n;
     drawrable = arraynew(sizeof(char *));
     n = arraynum(&(Menulocal.drawrable));
     for (i = 0; i < n; i++) {
@@ -571,7 +410,6 @@ CmOutputPrinter(int select_file, int show_dialog)
 {
   GtkPrintOperation *print;
   GtkPrintOperationResult res;
-  char buf[MESSAGE_BUF_SIZE];
   struct objlist *graobj, *g2wobj;
   int id, g2wid, g2woid, opt;
   N_VALUE *g2winst;
@@ -662,15 +500,10 @@ CmOutputPrinter(int select_file, int show_dialog)
     opt = GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG;
   }
 
-#ifdef __WIN64__  
-  gtk_widget_set_visible(NgraphApp.Viewer.side_pane1, FALSE);
-#endif
   res = gtk_print_operation_run(print, opt, GTK_WINDOW(TopLevel), &error);
-#ifdef __WIN64__  
-  gtk_widget_set_visible(NgraphApp.Viewer.side_pane1, Menulocal.sidebar);
-#endif
 
   if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
+    char buf[MESSAGE_BUF_SIZE];
     snprintf(buf, sizeof(buf), _("Printing error: %s"), error->message);
     message_box(NULL, buf, _("Print"), RESPONS_ERROR);
     g_error_free(error);
@@ -689,58 +522,6 @@ CmOutputPrinter(int select_file, int show_dialog)
   }
 }
 
-void
-CmOutputDriver(void)
-{
-  struct objlist *graobj, *g2wobj;
-  int id, g2wid, g2woid;
-  N_VALUE *g2winst;
-  int ret;
-  struct savedstdio stdio;
-
-  if (Menulock || Globallock)
-    return;
-
-  if (Menulocal.select_data && ! SetFileHidden())
-    return;
-
-  FileAutoScale();
-  AdjustAxis();
-
-  graobj = chkobject("gra");
-  if (graobj == NULL)
-    return;
-
-  g2wobj = chkobject("gra2prn");
-  if (g2wobj == NULL)
-    return;
-
-  g2wid = newobj(g2wobj);
-  if (g2wid < 0)
-    return;
-
-  DriverDialog(&DlgDriver, g2wobj, g2wid);
-  ret = DialogExecute(TopLevel, &DlgDriver);
-
-  if (ret == IDOK) {
-    SetStatusBar(_("Spawning external driver."));
-    g2winst = chkobjinst(g2wobj, g2wid);
-    _getobj(g2wobj, "oid", g2winst, &g2woid);
-    id = newobj(graobj);
-    init_graobj(graobj, id, "gra2prn", g2woid);
-    ignorestdio(&stdio);
-    draw_gra(graobj, id, _("Drawing."), TRUE);
-    restorestdio(&stdio);
-    delobj(graobj, id);
-    ResetStatusBar();
-  }
-  delobj(g2wobj, g2wid);
-
-  if (Menulocal.select_data) {
-    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE, TRUE);
-  }
-}
-
 void
 CmOutputViewerB(void *wi, gpointer client_data)
 {
@@ -1019,9 +800,7 @@ CmOutputEMF(int type)
   struct objlist *graobj, *g2wobj;
   int id, g2wid, g2woid;
   N_VALUE *g2winst;
-  int ret;
-  char *title, *ext_str;
-  char *file, *tmp;
+  char *title, *file;
 
   if (Menulock || Globallock)
     return;
@@ -1030,6 +809,8 @@ CmOutputEMF(int type)
 
   file = NULL;
   if (type == MenuIdOutputEMFFile) {
+    int ret;
+    char *ext_str, *tmp;
     ext_str = "emf";
     tmp = get_base_ngp_name();
     ret = nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp,
