@@ -14,12 +14,17 @@
 #include <glib.h>
 
 #include "nhash.h"
+#include "odata.h"
 
+#include "math_scanner.h"
+#include "math_parser.h"
 #include "math_expression.h"
 #include "math_function.h"
 
 static int calc(MathExpression *exp, MathValue *val);
 static MathExpression *optimize(MathExpression *exp, int *err);
+static const char *get_cstring_from_expression(MathExpression *exp);
+static int math_expression_kind_of_string(MathExpression *exp);
 
 #define CALC_EXPRESSION(e, v)				\
   ((e->type == MATH_EXPRESSION_TYPE_DOUBLE) ?		\
@@ -130,6 +135,7 @@ register_arg(MathExpression *func, const char *arg_name, enum MATH_FUNCTION_ARG_
 {
   switch (type) {
   case MATH_FUNCTION_ARG_TYPE_DOUBLE:
+  case MATH_FUNCTION_ARG_TYPE_VARIABLE:
     if (math_equation_check_var(func->equation, arg_name) >= 0) {
       /* the variable already exist */
       return 1;
@@ -141,10 +147,30 @@ register_arg(MathExpression *func, const char *arg_name, enum MATH_FUNCTION_ARG_
       /* the array already exist */
       return 1;
     }
-    math_equation_add_array(func->equation, arg_name);
+    math_equation_add_array(func->equation, arg_name, FALSE);
+    break;
+  case MATH_FUNCTION_ARG_TYPE_STRING_ARRAY:
+    if (math_equation_check_string_array(func->equation, arg_name) >= 0) {
+      /* the array already exist */
+      return 1;
+    }
+    math_equation_add_array(func->equation, arg_name, TRUE);
+    break;
+  case MATH_FUNCTION_ARG_TYPE_STRING:
+  case MATH_FUNCTION_ARG_TYPE_STRING_VARIABLE:
+    if (math_equation_check_string_var(func->equation, arg_name) >= 0) {
+      /* the string variable already exists */
+      return 1;
+    }
+    math_equation_add_var_string(func->equation, arg_name);
     break;
   case MATH_FUNCTION_ARG_TYPE_PROC:
     return 1;
+  case MATH_FUNCTION_ARG_TYPE_VARIABLE_COMMON:
+  case MATH_FUNCTION_ARG_TYPE_ARRAY_COMMON:
+    /* this function is called only at the user function definition. */
+    /* never reached */
+    return 1;
   }
 
   return 0;
@@ -173,17 +199,23 @@ static int
 func_set_arg_buf(MathExpression *func)
 {
   static struct math_func_arg_list *list;
-  int n, i, have_array;
+  int n, i, save_arg_type;
   enum MATH_FUNCTION_ARG_TYPE *arg_type_buf;
 
-  have_array = 0;
+  save_arg_type = 0;
   list = func->u.func.arg_list;
   for (n = 0; list; n++) {
     switch (list->type) {
     case MATH_FUNCTION_ARG_TYPE_DOUBLE:
       break;
+    case MATH_FUNCTION_ARG_TYPE_VARIABLE:
+    case MATH_FUNCTION_ARG_TYPE_STRING:
+    case MATH_FUNCTION_ARG_TYPE_STRING_VARIABLE:
+    case MATH_FUNCTION_ARG_TYPE_VARIABLE_COMMON:
     case MATH_FUNCTION_ARG_TYPE_ARRAY:
-      have_array = 1;
+    case MATH_FUNCTION_ARG_TYPE_STRING_ARRAY:
+    case MATH_FUNCTION_ARG_TYPE_ARRAY_COMMON:
+      save_arg_type = 1;
       break;
     case MATH_FUNCTION_ARG_TYPE_PROC:
       return 1;
@@ -192,7 +224,7 @@ func_set_arg_buf(MathExpression *func)
     list = list->next;
   }
 
-  if (have_array) {
+  if (save_arg_type) {
     arg_type_buf = g_malloc(sizeof(*arg_type_buf) * n);
     if (arg_type_buf == NULL)
       return 1;
@@ -247,7 +279,7 @@ math_function_expression_register_arg(MathExpression *func)
 int
 math_function_expression_set_function(MathEquation *eq, MathExpression *func, const char *name, MathExpression *exp)
 {
-  int anum, vnum;
+  int anum, str_anum, vnum, snum;
 
   func->u.func.exp = exp;
 
@@ -261,9 +293,11 @@ math_function_expression_set_function(MathEquation *eq, MathExpression *func, co
 
   func->u.func.fprm->base_usr = func;
   func->u.func.fprm->opt_usr = NULL;
-  math_equation_finish_user_func_definition(eq, &vnum, &anum);
+  math_equation_finish_user_func_definition(eq, &vnum, &anum, &str_anum, &snum);
   func->u.func.local_num = vnum;
+  func->u.func.local_string_num = snum;
   func->u.func.local_array_num = anum;
+  func->u.func.local_string_array_num = str_anum;
 
   return 0;
 }
@@ -383,17 +417,42 @@ math_variable_expression_new(MathEquation *eq, const char *name, int *err)
 }
 
 MathExpression *
-math_array_expression_new(MathEquation *eq, const char *name, MathExpression *operand, int *err)
+math_string_variable_expression_new(MathEquation *eq, const char *str, int *err)
 {
   MathExpression *exp;
   int i;
 
-  exp = math_expression_new(MATH_EXPRESSION_TYPE_ARRAY, eq, err);
+  exp = math_expression_new(MATH_EXPRESSION_TYPE_STRING_VARIABLE, eq, err);
   if (exp == NULL) {
     return NULL;
   }
 
-  i = math_equation_add_array(eq, name);
+  i = math_equation_add_var_string(eq, str);
+  if (i < 0) {
+    *err = MATH_ERROR_MEMORY;
+    math_expression_free(exp);
+    return NULL;
+  }
+
+  exp->u.index = i;
+
+  return exp;
+}
+
+MathExpression *
+math_array_expression_new(MathEquation *eq, const char *name, MathExpression *operand, int is_string, int *err)
+{
+  MathExpression *exp;
+  int i;
+  enum MATH_EXPRESSION_TYPE type;
+
+  type = (is_string) ? MATH_EXPRESSION_TYPE_STRING_ARRAY : MATH_EXPRESSION_TYPE_ARRAY;
+  exp = math_expression_new(type, eq, err);
+  if (exp == NULL) {
+    return NULL;
+  }
+
+  i = math_equation_add_array(eq, name, is_string);
   if (i < 0) {
     *err = MATH_ERROR_MEMORY;
     math_expression_free(exp);
@@ -417,7 +476,30 @@ math_array_argument_expression_new(MathEquation *eq, const char *name, int *err)
     return NULL;
   }
 
-  i = math_equation_add_array(eq, name);
+  i = math_equation_add_array(eq, name, FALSE);
+  if (i < 0) {
+    *err = MATH_ERROR_MEMORY;
+    math_expression_free(exp);
+    return NULL;
+  }
+
+  exp->u.array.index = i;
+
+  return exp;
+}
+
+MathExpression *
+math_string_array_argument_expression_new(MathEquation *eq, const char *name, int *err)
+{
+  MathExpression *exp;
+  int i;
+
+  exp = math_expression_new(MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT, eq, err);
+  if (exp == NULL) {
+    return NULL;
+  }
+
+  i = math_equation_add_array(eq, name, TRUE);
   if (i < 0) {
     *err = MATH_ERROR_MEMORY;
     math_expression_free(exp);
@@ -438,18 +520,102 @@ math_function_get_arg_type_num(struct math_function_parameter *fprm)
   return fprm->argc;
 }
 
+static int
+check_argument_sub(enum MATH_FUNCTION_ARG_TYPE arg_type, enum MATH_EXPRESSION_TYPE type)
+{
+  switch (arg_type) {
+  case MATH_FUNCTION_ARG_TYPE_DOUBLE:
+  case MATH_FUNCTION_ARG_TYPE_PROC:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
+      return 1;
+    default:
+      break;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_STRING:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_STRING:
+    case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_ARRAY:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_STRING_ARRAY:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_ARRAY_COMMON:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_VARIABLE:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_VARIABLE:
+    case MATH_EXPRESSION_TYPE_ARRAY:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_STRING_VARIABLE:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  case MATH_FUNCTION_ARG_TYPE_VARIABLE_COMMON:
+    switch (type) {
+    case MATH_EXPRESSION_TYPE_VARIABLE:
+    case MATH_EXPRESSION_TYPE_ARRAY:
+    case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+      break;
+    default:
+      return 1;
+    }
+    break;
+  }
+  return 0;
+}
+
 static int
 check_argument(struct math_function_parameter *fprm, int argc, MathExpression **argv)
 {
   int i, arg_type_num;
+  enum MATH_FUNCTION_ARG_TYPE arg_type;
 
   arg_type_num = math_function_get_arg_type_num(fprm);
   for (i = 0; i < argc; i++) {
-    if (fprm->arg_type && i < arg_type_num && fprm->arg_type[i] == MATH_FUNCTION_ARG_TYPE_ARRAY) {
-      if (argv[i]->type != MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT) {
-	return 1;
-      }
-    } else if (argv[i]->type == MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT) {
+    if (fprm->arg_type && i < arg_type_num) {
+      arg_type = fprm->arg_type[i];
+    } else {
+      arg_type = MATH_FUNCTION_ARG_TYPE_DOUBLE;
+    }
+    if (check_argument_sub(arg_type, argv[i]->type)) {
       return 1;
     }
   }
@@ -464,6 +630,7 @@ math_func_call_expression_new(MathEquation *eq, struct math_function_parameter *
   MathFunctionArgument *buf;
 
   if (check_argument(fprm, argc, argv)) {
+    *err = MATH_ERROR_INVALID_ARG;
     return NULL;
   }
 
@@ -471,6 +638,7 @@ math_func_call_expression_new(MathEquation *eq, struct math_function_parameter *
   if (argc > 0) {
     buf = g_malloc(sizeof(*buf) * argc);
     if (buf == NULL) {
+      *err = MATH_ERROR_MEMORY;
       return NULL;
     }
   }
@@ -550,6 +718,210 @@ math_double_expression_new(MathEquation *eq, const MathValue *val, int *err)
   return exp;
 }
 
+static int
+check_expand(MathEquation *eq, MathStringExpression *str, int *err)
+{
+  char *ptr, *eqn, error;
+  int in_variable, start, end, i;
+  struct embedded_expression variable;
+  MathExpression *exp;
+
+  error = FALSE;
+  in_variable = FALSE;
+  ptr = str->string;
+  for (i = 0; ptr[i]; i++) {
+    switch (ptr[i]) {
+    case MATH_VARIABLE_EXPAND_PREFIX:
+      if (ptr[i + 1] == '{') {
+	in_variable = TRUE;
+	start = i;
+	i++;
+	continue;
+      }
+      break;
+    case '}':
+      if (in_variable) {
+	end = i;
+	variable.start = start;
+	variable.end = end;
+	eqn = g_strndup(ptr + start + 2, end - start - 2);
+	if (eqn == NULL) {
+	  *err = MATH_ERROR_MEMORY;
+	  error =TRUE;
+	  goto End;
+	}
+	exp = math_parser_parse(eqn, eq, err);
+	if (exp == NULL) {
+	  error = TRUE;
+	  g_free(eqn);
+	  goto End;
+	}
+	variable.exp = exp;
+	if (arrayadd(str->variables, &variable) == NULL) {
+	  *err = MATH_ERROR_MEMORY;
+	  error = TRUE;
+	  math_expression_free(exp);
+	}
+	g_free(eqn);
+	in_variable = FALSE;
+	if (error) {
+	  goto End;
+	}
+      }
+      break;
+    }
+  }
+  if (! error && in_variable) {
+    *err = MATH_ERROR_UNTERMINATED_STRING;
+    error = TRUE;
+  }
+ End:
+  return error;
+}
+
+MathExpression *
+math_string_expression_new(MathEquation *eq, const char *str, int expand, int *err)
+{
+  MathExpression *exp;
+
+  exp = math_expression_new(MATH_EXPRESSION_TYPE_STRING, eq, err);
+  if (exp == NULL)
+    return NULL;
+
+  exp->u.str.variables = NULL;
+  exp->u.str.expanded = NULL;
+  exp->u.str.string = g_strdup(str);
+  if (exp->u.str.string == NULL) {
+    *err = MATH_ERROR_MEMORY;
+    math_expression_free(exp);
+    return NULL;
+  }
+  if (! expand) {
+    return exp;
+  }
+
+  exp->u.str.variables = arraynew(sizeof(struct embedded_expression));
+  if (exp->u.str.variables == NULL) {
+    *err = MATH_ERROR_MEMORY;
+    math_expression_free(exp);
+    return NULL;
+  }
+  if (check_expand(eq, &exp->u.str, err)) {
+    math_expression_free(exp);
+    return NULL;
+  }
+  if (arraynum(exp->u.str.variables) == 0) {
+    arrayfree(exp->u.str.variables);
+    exp->u.str.variables = NULL;
+  }
+  if (exp->u.str.variables) {
+    exp->u.str.expanded = g_string_new("");
+    if (exp->u.str.expanded == NULL) {
+      arrayfree(exp->u.str.variables);
+      exp->u.str.variables = NULL;
+    }
+  }
+  return exp;
+}
+
+static const char *
+math_expression_get_string(MathExpression *expression)
+{
+  int i, n;
+  struct embedded_expression *var;
+  GString *gstr;
+  char *ptr;
+  int top;
+  MathStringExpression *str_exp;
+
+  if (expression == NULL) {
+    return NULL;
+  }
+  str_exp = &(expression->u.str);
+  if (str_exp->variables == NULL) {
+    return str_exp->string;
+  }
+  n = arraynum(str_exp->variables);
+  if (n < 1) {
+    return str_exp->string;
+  }
+  gstr = str_exp->expanded;
+  g_string_set_size(gstr, 0);
+  ptr = str_exp->string;
+  top = 0;
+  var = arraydata(str_exp->variables);
+  for (i = 0; i < n; i++) {
+    MathExpression *exp;
+    MathValue val;
+    int len;
+    const char *cstr;
+    len = var[i].start - top;
+    g_string_append_len(gstr, ptr + top, len);
+    exp = var[i].exp;
+    switch (exp->type) {
+    case MATH_EXPRESSION_TYPE_STRING:
+      g_string_append(gstr, exp->u.str.string);
+      break;
+    case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+      cstr = get_cstring_from_expression(exp);
+      if (cstr) {
+	g_string_append(gstr, cstr);
+      }
+      break;
+    default:
+      if (math_expression_calculate(exp, &val) == 0) {
+	if (val.type == MATH_VALUE_NORMAL) {
+	  g_string_append_printf(gstr, "%G", val.val);
+	} else {
+	  const char *str;
+	  str = math_special_value_to_string(&val);
+	  if (str) {
+	    g_string_append_c(gstr, '[');
+	    g_string_append(gstr, str);
+	    g_string_append_c(gstr, ']');
+	  }
+	}
+      }
+    }
+    top = var[i].end + 1;
+    if (i == n - 1) {
+      g_string_append(gstr, ptr + var[i].end + 1);
+    }
+  }
+  return gstr->str;
+}
+
+static void
+free_expand_variables(struct narray *array)
+{
+  int i, n;
+  struct embedded_expression *data;
+  if (array == NULL) {
+    return;
+  }
+  n = arraynum(array);
+  data = arraydata(array);
+  for (i = 0; i < n; i++) {
+    math_expression_free(data[i].exp);
+  }
+}
+
+static void
+math_string_expression_free(MathStringExpression *exp)
+{
+  if (exp->string) {
+    g_free(exp->string);
+  }
+  if (exp->expanded) {
+    g_string_free(exp->expanded, TRUE);
+  }
+  if (exp->variables) {
+    free_expand_variables(exp->variables);
+    arrayfree(exp->variables);
+  }
+}
+
 MathExpression *
 math_eoeq_expression_new(MathEquation *eq, int *err)
 {
@@ -586,6 +958,7 @@ math_expression_free_sub(MathExpression *exp)
     math_expression_free(exp->u.bin.right);
     break;
   case MATH_EXPRESSION_TYPE_ASSIGN:
+  case MATH_EXPRESSION_TYPE_STRING_ASSIGN:
     math_expression_free(exp->u.assign.left);
     math_expression_free(exp->u.assign.right);
     break;
@@ -604,20 +977,31 @@ math_expression_free_sub(MathExpression *exp)
     g_free(exp->u.func_call.buf);
     break;
   case MATH_EXPRESSION_TYPE_MINUS:
+  case MATH_EXPRESSION_TYPE_NOT:
   case MATH_EXPRESSION_TYPE_FACT:
     math_expression_free(exp->u.unary.operand);
     break;
   case MATH_EXPRESSION_TYPE_ARRAY:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
     math_expression_free(exp->u.array.operand);
     break;
   case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
   case MATH_EXPRESSION_TYPE_CONST:
   case MATH_EXPRESSION_TYPE_VARIABLE:
     break;
+  case MATH_EXPRESSION_TYPE_STRING:
+    math_string_expression_free(&exp->u.str);
+    break;
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    break;
   case MATH_EXPRESSION_TYPE_DOUBLE:
   case MATH_EXPRESSION_TYPE_PRM:
   case MATH_EXPRESSION_TYPE_EOEQ:
     break;
+  case MATH_EXPRESSION_TYPE_BLOCK:
+    math_expression_free(exp->u.exp);
+    break;
   }
 
   g_free(exp);
@@ -821,6 +1205,78 @@ factorial(unsigned int n)
   return HUGE_VAL;
 }
 
+static int
+set_string_argument(MathFunctionCallExpression *exp, int i)
+{
+  const char *str;
+
+  if (! math_expression_kind_of_string(exp->argv[i])) {
+    return 1;
+  }
+  str = get_cstring_from_expression(exp->argv[i]);
+  if (str == NULL) {
+    return 1;
+  }
+  exp->buf[i].cstr = str;
+  return 0;
+}
+
+static int
+set_variable_argument(MathFunctionCallExpression *exp, MathEquation *eq, int i)
+{
+  MathValue v, *ptr;
+
+  switch (exp->argv[i]->type) {
+  case MATH_EXPRESSION_TYPE_ARRAY:
+    if (CALC_EXPRESSION(exp->argv[i]->u.array.operand, v)) {
+      return 1;
+    }
+    if (v.type != MATH_VALUE_NORMAL) {
+      return 1;
+    }
+    ptr = math_equation_get_array_ptr(eq, exp->argv[i]->u.array.index, v.val);
+    break;
+  case MATH_EXPRESSION_TYPE_VARIABLE:
+    ptr = math_equation_get_var_ptr(eq, exp->argv[i]->u.index);
+    break;
+  default:
+    return 1;
+  }
+  exp->buf[i].variable.data.vptr = ptr;
+  exp->buf[i].variable.type = DATA_TYPE_VALUE;
+  return 0;
+}
+
+static int
+set_string_variable_argument(MathFunctionCallExpression *exp, MathEquation *eq, int i)
+{
+  MathValue v;
+  GString *gstr;
+
+  switch (exp->argv[i]->type) {
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+    if (CALC_EXPRESSION(exp->argv[i]->u.array.operand, v)) {
+      return 1;
+    }
+    if (v.type != MATH_VALUE_NORMAL) {
+      return 1;
+    }
+    gstr = math_equation_get_array_str(eq, exp->argv[i]->u.array.index, v.val);
+    break;
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    math_equation_get_string_var(eq, exp->argv[i]->u.index, &gstr);
+    break;
+  default:
+    return 1;
+  }
+  if (gstr == NULL) {
+    return 1;
+  }
+  exp->buf[i].variable.type = DATA_TYPE_STRING;
+  exp->buf[i].variable.data.str = gstr;
+  return 0;
+}
+
 static int
 call_func(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *val)
 {
@@ -839,12 +1295,45 @@ call_func(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *val)
   for (i = 0; i < n; i++) {
     if (type && i < arg_type_num) {
       switch (type[i]) {
-      case MATH_FUNCTION_ARG_TYPE_ARRAY:
-	if (exp->argv[i]->type != MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT) {
+      case MATH_FUNCTION_ARG_TYPE_VARIABLE:
+	if (set_variable_argument(exp, eq, i)) {
 	  val->type = MATH_VALUE_ERROR;
 	  return 1;
 	}
-	exp->buf[i].idx = exp->argv[i]->u.array.index;
+	break;
+      case MATH_FUNCTION_ARG_TYPE_VARIABLE_COMMON:
+        switch (exp->argv[i]->type) {
+        case MATH_EXPRESSION_TYPE_ARRAY:
+        case MATH_EXPRESSION_TYPE_VARIABLE:
+          if (set_variable_argument(exp, eq, i)) {
+            val->type = MATH_VALUE_ERROR;
+            return 1;
+          }
+          break;
+        case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+        case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+          if (set_string_variable_argument(exp, eq, i)) {
+            val->type = MATH_VALUE_ERROR;
+            return 1;
+          }
+          break;
+        default:
+          val->type = MATH_VALUE_ERROR;
+          return 1;
+          break;
+        }
+        break;
+      case MATH_FUNCTION_ARG_TYPE_ARRAY:
+        switch (exp->argv[i]->type) {
+        case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+          exp->buf[i].array.array_type = DATA_TYPE_VALUE;
+          exp->buf[i].array.idx = exp->argv[i]->u.array.index;
+          break;
+        default:
+	  val->type = MATH_VALUE_ERROR;
+	  return 1;
+          break;
+        }
 	break;
       case MATH_FUNCTION_ARG_TYPE_PROC:
 	if (exp->argv[i]->type == MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT) {
@@ -861,6 +1350,44 @@ call_func(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *val)
 	if (CALC_EXPRESSION(exp->argv[i], exp->buf[i].val))
 	  return 1;
 	break;
+      case MATH_FUNCTION_ARG_TYPE_STRING:
+	if (set_string_argument(exp, i)) {
+	  val->type = MATH_VALUE_ERROR;
+	  return 1;
+	}
+	break;
+      case MATH_FUNCTION_ARG_TYPE_STRING_VARIABLE:
+	if (set_string_variable_argument(exp, eq, i)) {
+	  val->type = MATH_VALUE_ERROR;
+	  return 1;
+	}
+	break;
+      case MATH_FUNCTION_ARG_TYPE_STRING_ARRAY:
+        switch (exp->argv[i]->type) {
+        case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
+          exp->buf[i].array.array_type = DATA_TYPE_STRING;
+          exp->buf[i].array.idx = exp->argv[i]->u.array.index;
+          break;
+        default:
+	  val->type = MATH_VALUE_ERROR;
+	  return 1;
+          break;
+        }
+	break;
+      case MATH_FUNCTION_ARG_TYPE_ARRAY_COMMON:
+        switch (exp->argv[i]->type) {
+        case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+          exp->buf[i].array.array_type = DATA_TYPE_VALUE;
+          exp->buf[i].array.idx = exp->argv[i]->u.array.index;
+          break;
+        case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
+          exp->buf[i].array.array_type = DATA_TYPE_STRING;
+          exp->buf[i].array.idx = exp->argv[i]->u.array.index;
+          break;
+        default:
+	  return 1;
+          break;
+        }
       }
     } else if (CALC_EXPRESSION(exp->argv[i], exp->buf[i].val)) {
       return 1;
@@ -906,6 +1433,8 @@ optimize_usr_function(MathExpression *exp,int *err)
   new_exp->u.func.argc = exp->u.func.argc;
   new_exp->u.func.local_num = exp->u.func.local_num;
   new_exp->u.func.local_array_num = exp->u.func.local_array_num;
+  new_exp->u.func.local_string_num = exp->u.func.local_string_num;
+  new_exp->u.func.local_string_array_num = exp->u.func.local_string_array_num;
   new_exp->u.func.exp = math_expression_optimize(exp->u.func.exp, err);
   new_exp->u.func.fprm = exp->u.func.fprm;
   new_exp->u.func.arg_list = NULL;
@@ -1108,10 +1637,6 @@ optimize_assign_expression(MathExpression *exp, int *err)
     return NULL;
   }
 
-  if (left->type == MATH_EXPRESSION_TYPE_DOUBLE &&
-      right->type == MATH_EXPRESSION_TYPE_DOUBLE) {
-    new_exp = reduce_expression(new_exp, err);
-  }
   return new_exp;
 }
 
@@ -1188,8 +1713,7 @@ optimize_div_expression(MathExpression *exp, int *err)
       math_expression_free(new_exp);
       new_exp = left;
     } else if (right->u.value.val == 0.0) {
-      right->u.value.val = right->u.value.val;
-      new_exp->type = MATH_EXPRESSION_TYPE_DIV;
+      new_exp->type = MATH_EXPRESSION_TYPE_DIV; /* noting to do (a run-time error will be caused) */
     } else {
       right->u.value.val = 1.0 / right->u.value.val;
       new_exp->type = MATH_EXPRESSION_TYPE_MUL;
@@ -1242,12 +1766,85 @@ optimize_array_expression(MathExpression *exp, int *err)
   return new_exp;
 }
 
+static MathExpression *
+optimize_block_expression(MathExpression *exp, int *err)
+{
+  MathExpression *operand, *new_exp;
+  operand = math_expression_optimize(exp->u.exp, err);
+  if (operand == NULL) {
+    return NULL;
+  }
+  new_exp = math_expression_new(exp->type, exp->equation, err);
+  if (new_exp == NULL) {
+    return NULL;
+  }
+  new_exp->u.exp = operand;
+  return new_exp;
+}
+
+static MathExpression *
+optimize_string_expression(MathExpression *exp, int *err)
+{
+  MathExpression *new_exp;
+  int i, n, is_fail;
+  struct embedded_expression *org_data, *new_data;
+
+  new_exp = math_expression_new(MATH_EXPRESSION_TYPE_STRING, exp->equation, err);
+  if (new_exp == NULL) {
+    return NULL;
+  }
+  new_exp->u.str.expanded = NULL;
+  new_exp->u.str.variables = NULL;
+  new_exp->u.str.string = g_strdup(exp->u.str.string);
+  if (new_exp->u.str.string == NULL) {
+    g_free(new_exp);
+    return NULL;
+  }
+
+  if (exp->u.str.expanded == NULL) {
+    return new_exp;
+  }
+
+  new_exp->u.str.expanded = g_string_new("");
+  if (new_exp->u.str.expanded == NULL) {
+    math_expression_free(new_exp);
+    return NULL;
+  }
+
+  new_exp->u.str.variables = arraydup(exp->u.str.variables);
+  if (new_exp->u.str.variables == NULL) {
+    math_expression_free(new_exp);
+    return NULL;
+  }
+
+  is_fail = FALSE;
+  n = arraynum(new_exp->u.str.variables);
+  new_data = arraydata(new_exp->u.str.variables);
+  org_data = arraydata(exp->u.str.variables);
+  for (i = 0; i < n; i++) {
+    new_data[i].exp = optimize(org_data[i].exp, err);
+    if (new_data[i].exp == NULL) {
+      is_fail = TRUE;
+    }
+  }
+
+  if (is_fail) {
+    math_expression_free(new_exp);
+    return NULL;
+  }
+
+  return new_exp;
+}
+
 static MathExpression *
 optimize(MathExpression *exp, int *err)
 {
   MathExpression *new_exp;
 
   switch (exp->type) {
+  case MATH_EXPRESSION_TYPE_BLOCK:
+    new_exp = optimize_block_expression(exp, err);
+    break;
   case MATH_EXPRESSION_TYPE_OR:
     new_exp = optimize_or_expression(exp, err);
     break;
@@ -1267,6 +1864,7 @@ optimize(MathExpression *exp, int *err)
     new_exp = optimize_bin_expression(exp, err);
     break;
   case MATH_EXPRESSION_TYPE_ASSIGN:
+  case MATH_EXPRESSION_TYPE_STRING_ASSIGN:
     new_exp = optimize_assign_expression(exp, err);
     break;
   case MATH_EXPRESSION_TYPE_MUL:
@@ -1276,6 +1874,7 @@ optimize(MathExpression *exp, int *err)
     new_exp = optimize_div_expression(exp, err);
     break;
   case MATH_EXPRESSION_TYPE_MINUS:
+  case MATH_EXPRESSION_TYPE_NOT:
   case MATH_EXPRESSION_TYPE_FACT:
     new_exp = optimize_una_expression(exp, err);
     break;
@@ -1296,6 +1895,7 @@ optimize(MathExpression *exp, int *err)
     }
     break;
   case MATH_EXPRESSION_TYPE_VARIABLE:
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
     new_exp = math_expression_new(exp->type, exp->equation, err);
     if (new_exp)
       new_exp->u.index = exp->u.index;
@@ -1313,15 +1913,20 @@ optimize(MathExpression *exp, int *err)
     new_exp = math_eoeq_expression_new(exp->equation, err);
     break;
   case MATH_EXPRESSION_TYPE_ARRAY:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
     new_exp = optimize_array_expression(exp, err);
     break;
   case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
     new_exp = math_expression_new(exp->type, exp->equation, err);
     if (new_exp) {
       new_exp->u.array.index = exp->u.array.index;
     }
     break;
-  default:
+  case MATH_EXPRESSION_TYPE_STRING:
+    new_exp = optimize_string_expression(exp, err);
+    break;
+  case MATH_EXPRESSION_TYPE_CONST_DEF:
     new_exp = NULL; /* never reached */
   }
 
@@ -1416,6 +2021,139 @@ set_val_to_array(MathExpression *exp, MathValue *val, enum MATH_OPERATOR_TYPE op
     break;								\
   }
 
+static char *
+val2str(MathValue *val)
+{
+  char *str;
+  const char *tmp;
+
+  if (val == NULL) {
+    return NULL;
+  }
+
+  tmp = math_special_value_to_string(val);
+  if (tmp == NULL) {
+    str = g_strdup_printf("%G", val->val);
+  } else {
+    str = g_strdup(tmp);
+  }
+  return str;
+}
+
+static int
+assign_string(MathExpression *exp)
+{
+  const char *str;
+  char *tmp;
+  MathValue operand;
+  MathExpression *left, *right;
+
+  tmp = NULL;
+  right = exp->u.assign.right;
+  if (math_expression_kind_of_string(right)) {
+    str = get_cstring_from_expression(right);
+  } else {
+    if (CALC_EXPRESSION(right, operand)) {
+      return 1;
+    }
+    str = tmp = val2str(&operand);
+  }
+  left = exp->u.assign.left;
+  if (left->type == MATH_EXPRESSION_TYPE_STRING_VARIABLE) {
+    if (math_equation_set_var_string(exp->equation, left->u.index, str)) {
+      return 1;
+    }
+  } else {
+    MathValue v;
+    int i;
+    if (exp->u.assign.op != MATH_OPERATOR_TYPE_ASSIGN) {
+      return 1;
+    }
+    if (CALC_EXPRESSION(left->u.array.operand, v)) {
+      return 1;
+    }
+    i = v.val;
+    if (math_equation_set_array_str(exp->equation, left->u.array.index, i, str)) {
+      return 1;
+    }
+  }
+  if (tmp) {
+    g_free(tmp);
+  }
+  return 0;
+}
+
+static int
+math_expression_kind_of_string(MathExpression *exp)
+{
+  switch (exp->type) {
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+  case MATH_EXPRESSION_TYPE_STRING:
+    return TRUE;
+  default:
+    return FALSE;
+  }
+  return FALSE;
+}
+
+static const char *
+get_cstring_from_expression(MathExpression *exp)
+{
+  const char *cstr;
+  GString *gstr;
+  MathValue val;
+
+  if (exp == NULL) {
+    return NULL;
+  }
+
+  cstr = NULL;
+  switch (exp->type) {
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    math_equation_get_string_var(exp->equation, exp->u.index, &gstr);
+    if (gstr && gstr->str) {
+      cstr = gstr->str;
+    }
+    break;
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+    if (CALC_EXPRESSION(exp->u.array.operand, val) == 0) {
+      if (val.type == MATH_VALUE_NORMAL) {
+	cstr = math_equation_get_array_cstr(exp->equation, exp->u.array.index, val.val);
+      }
+    }
+    break;
+  case MATH_EXPRESSION_TYPE_STRING:
+    cstr = math_expression_get_string(exp);
+    break;
+  default:
+    break;
+  }
+  return cstr;
+}
+
+const char *
+math_expression_get_cstring(MathExpression *exp)
+{
+  return get_cstring_from_expression(exp);
+}
+
+static int
+compare_string(MathExpression *left, MathExpression *right, int equal, MathValue *val)
+{
+  const char *lstr, *rstr;
+  int result;
+
+  lstr = get_cstring_from_expression(left);
+  rstr = get_cstring_from_expression(right);
+  if (left == NULL || right == NULL) {
+    return 1;
+  }
+  result = g_strcmp0(lstr, rstr) ? 1 : 0;
+  val->val = (equal) ? ! result : result;
+  return 0;
+}
+
 static int
 calc(MathExpression *exp, MathValue *val)
 {
@@ -1424,6 +2162,12 @@ calc(MathExpression *exp, MathValue *val)
   val->type = MATH_VALUE_NORMAL;
 
   switch (exp->type) {
+  case MATH_EXPRESSION_TYPE_BLOCK:
+    if (math_expression_calculate(exp->u.exp, &operand)) {
+      return 1;
+    }
+    *val = operand;
+    break;
   case MATH_EXPRESSION_TYPE_OR:
     if (CALC_EXPRESSION(exp->u.bin.left, left)) {
       return 1;
@@ -1453,6 +2197,13 @@ calc(MathExpression *exp, MathValue *val)
     *val = right;
     break;
   case MATH_EXPRESSION_TYPE_EQ:
+    if (math_expression_kind_of_string(exp->u.bin.left) &&
+	math_expression_kind_of_string(exp->u.bin.right)) {
+      if (compare_string(exp->u.bin.left, exp->u.bin.right, TRUE, val)) {
+	return 1;
+      }
+      break;
+    }
     if (CALC_EXPRESSION(exp->u.bin.left, left)) {
       return 1;
     }
@@ -1464,6 +2215,13 @@ calc(MathExpression *exp, MathValue *val)
     val->val = (left.val == right.val);
     break;
   case MATH_EXPRESSION_TYPE_NE:
+    if (math_expression_kind_of_string(exp->u.bin.left) &&
+	math_expression_kind_of_string(exp->u.bin.right)) {
+      if (compare_string(exp->u.bin.left, exp->u.bin.right, FALSE, val)) {
+	return 1;
+      }
+      break;
+    }
     if (CALC_EXPRESSION(exp->u.bin.left, left)) {
       return 1;
     }
@@ -1605,6 +2363,13 @@ calc(MathExpression *exp, MathValue *val)
     MATH_CHECK_VAL(val, operand);
     val->val = - operand.val;
     break;
+  case MATH_EXPRESSION_TYPE_NOT:
+    if (CALC_EXPRESSION(exp->u.unary.operand, operand)) {
+      return 1;
+    }
+    MATH_CHECK_VAL(val, operand);
+    val->val = ! operand.val;
+    break;
   case MATH_EXPRESSION_TYPE_FACT:
     if (CALC_EXPRESSION(exp->u.unary.operand, operand)) {
       return 1;
@@ -1653,10 +2418,30 @@ calc(MathExpression *exp, MathValue *val)
     if (CALC_EXPRESSION(exp->u.array.operand, operand)) {
       return 1;
     }
+    if (operand.type != MATH_VALUE_NORMAL) {
+      return 1;
+    }
     if (math_equation_get_array_val(exp->equation, exp->u.array.index, operand.val, val)) {
       return 1;
     }
     break;
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY:
+    if (CALC_EXPRESSION(exp->u.array.operand, operand)) {
+      return 1;
+    }
+    if (operand.type != MATH_VALUE_NORMAL) {
+      return 1;
+    }
+    val->val = 0;
+    {
+      const char *str;
+      str = math_equation_get_array_cstr(exp->equation, exp->u.array.index, operand.val);
+      if (str == NULL) {
+        return 1;
+      }
+      n_strtod(str, val);
+    }
+    break;
   case MATH_EXPRESSION_TYPE_PRM:
     if (exp->u.prm.prm->data == NULL) {
       return 1;
@@ -1667,7 +2452,34 @@ calc(MathExpression *exp, MathValue *val)
     val->val = 0;
     break;
   case MATH_EXPRESSION_TYPE_ARRAY_ARGUMENT:
+  case MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT:
     return 1;
+  case MATH_EXPRESSION_TYPE_STRING_VARIABLE:
+    val->val = 0;
+    {
+      GString *gstr;
+      math_equation_get_string_var(exp->equation, exp->u.index, &gstr);
+      if (gstr && gstr->str) {
+        n_strtod(gstr->str, val);
+      }
+    }
+    break;
+  case MATH_EXPRESSION_TYPE_STRING:
+    val->val = 0;
+    {
+      const char *str;
+      str = math_expression_get_string(exp);
+      if (str) {
+	n_strtod(str, val);
+      }
+    }
+    break;
+  case MATH_EXPRESSION_TYPE_STRING_ASSIGN:
+    if (assign_string(exp)) {
+      return 1;
+    }
+    val->val = 0;	    /* a string is always evaluated as zero */
+    break;
   case MATH_EXPRESSION_TYPE_FUNC:
   case MATH_EXPRESSION_TYPE_CONST_DEF:
     /* never reached */
@@ -1678,6 +2490,70 @@ calc(MathExpression *exp, MathValue *val)
   return 0;
 }
 
+enum DATA_TYPE
+math_expression_get_variable_type_from_argument(MathFunctionCallExpression *exp, int i)
+{
+  return exp->buf[i].variable.type;
+}
+
+const char *
+math_expression_get_string_from_argument(MathFunctionCallExpression *exp, int i)
+{
+  return exp->buf[i].cstr;
+}
+
+GString *
+math_expression_get_string_variable_from_argument(MathFunctionCallExpression *exp, int i)
+{
+  if (exp->buf[i].variable.type != DATA_TYPE_STRING) {
+    return NULL;
+  }
+  return exp->buf[i].variable.data.str;
+}
+
+int
+math_function_call_expression_get_variable(MathFunctionCallExpression *exp, int i, MathVariable *var)
+{
+  var->type = exp->buf[i].variable.type;
+  switch (var->type) {
+  case DATA_TYPE_VALUE:
+    var->type = DATA_TYPE_VALUE;
+    var->data.vptr = exp->buf[i].variable.data.vptr;
+    break;
+  case DATA_TYPE_STRING:
+    var->type = DATA_TYPE_STRING;
+    var->data.str = exp->buf[i].variable.data.str;
+    break;
+  }
+  return 0;
+}
+
+int
+math_variable_set_common_value(MathVariable *variable, MathCommonValue *val)
+{
+  if (variable->type != val->type) {
+    return 1;
+  }
+  switch (variable->type) {
+  case DATA_TYPE_VALUE:
+    *(variable->data.vptr) = val->data.val;
+    break;
+  case DATA_TYPE_STRING:
+    g_string_assign(variable->data.str, val->data.cstr);
+    break;
+  }
+  return 0;
+}
+
+MathValue *
+math_expression_get_variable_from_argument(MathFunctionCallExpression *exp, int i)
+{
+  if (exp->buf[i].variable.type != DATA_TYPE_VALUE) {
+    return NULL;
+  }
+  return exp->buf[i].variable.data.vptr;
+}
+
 int
 math_expression_calculate(MathExpression *exp, MathValue *val)
 {
