@@ -30,6 +30,7 @@
 
 #include "object.h"
 #include "nstring.h"
+#include "math/math_equation.h"
 
 #define NSTRLEN 0x400
 
@@ -112,7 +113,7 @@ nstrncat(char *po,char *s,size_t n)
 
   if (po==NULL) return NULL;
   if (s==NULL) return po;
-  for (i=0;(s[i]!='\0') && (i<n);i++)
+  for (i=0; (i < n) && (s[i] != '\0'); i++)
     if ((po=nstrccat(po,s[i]))==NULL) return NULL;
   return po;
 }
@@ -366,8 +367,6 @@ add_printf_formated_str(GString *str, const char *format, const char *arg, int *
 {
   int i, formated, pow;
   char *format2, *buf, *endptr;
-  int vi;
-  long long int vll;
   double vd;
 
   formated = FALSE;
@@ -383,12 +382,14 @@ add_printf_formated_str(GString *str, const char *format, const char *arg, int *
   switch (format[i]) {
   case 'd': case 'i': case 'o': case 'u': case 'x': case 'X':
     if (i > 2 && strncmp(format2 + i - 2, "ll", 2) == 0) {
+      long long int vll;
       vll = 0;
       if (arg) {
 	vll = strtod(arg, &endptr);
       }
       buf = g_strdup_printf(format2, vll);
-    }else {
+    } else {
+      int vi;
       vi = 0;
       if (arg) {
 	vi = strtod(arg, &endptr);
@@ -446,6 +447,97 @@ add_printf_formated_str(GString *str, const char *format, const char *arg, int *
   return formated;
 }
 
+static int
+check_value_type(GString *str, MathValue *mval)
+{
+  const char *s;
+  s = math_special_value_to_string(mval);
+  if (s == NULL) {
+    return 0;
+  }
+  g_string_append(str, s);
+  return 1;
+}
+
+int
+add_printf_formated_double(GString *str, const char *format, MathValue *mval, int *len)
+{
+  int i, formated, pow;
+  char *format2, *buf, *tmp;
+  int vi;
+  double val;
+
+  formated = FALSE;
+  format2 = get_printf_format_str(format, &i, &pow);
+  if (len) {
+    *len = i;
+  }
+  if (format2 == NULL) {
+    return formated;
+  }
+
+  if (check_value_type(str, mval)) {
+    return TRUE;
+  }
+  val = mval->val;
+  buf = NULL;
+  switch (format[i]) {
+  case 'd': case 'i': case 'o': case 'u': case 'x': case 'X':
+    if (i > 2 && strncmp(format2 + i - 2, "ll", 2) == 0) {
+      long long int vll;
+      vll = val;
+      buf = g_strdup_printf(format2, vll);
+    }else {
+      vi = val;
+      buf = g_strdup_printf(format2, vi);
+    }
+    formated = TRUE;
+    break;
+  case 'e': case 'E': case 'f': case 'F': case 'g': case 'G':
+    if (i > 2 && strncmp(format2 + i - 2, "ll", 2) == 0) {
+      break;
+    }
+    buf = g_strdup_printf(format2, val);
+    if (pow) {
+      char *new_buf;
+      new_buf = str_to_pow(buf);
+      if (new_buf) {
+	g_free(buf);
+	buf = new_buf;
+      }
+    }
+    formated = TRUE;
+    break;
+  case 's':
+    if (i > 1 && format2[i - 1] == 'l') {
+      break;
+    }
+    tmp = g_strdup_printf("%g", val);
+    if (tmp) {
+      buf = g_strdup_printf(format2, tmp);
+      g_free(tmp);
+    }
+    break;
+  case 'c':
+    if (i > 1 && format2[i - 1] == 'l') {
+      break;
+    }
+    vi = val;
+    buf = g_strdup_printf(format2, vi);
+    formated = TRUE;
+    break;
+  }
+
+  if (buf) {
+    g_string_append(str, buf);
+    g_free(buf);
+  }
+
+  g_free(format2);
+
+  return formated;
+}
+
 #ifdef COMPILE_UNUSED_FUNCTIONS
 char *
 getitok3(char **s,int *len,char *ifs)
@@ -494,3 +586,18 @@ getitok4(char **s,int *len,char *ifs)
   return po;
 }
 #endif /* COMPILE_UNUSED_FUNCTIONS */
+
+char *
+n_locale_to_utf8(const char *s)
+{
+  char *ustr;
+  if (s == NULL) {
+    return NULL;
+  }
+  if (g_utf8_validate(s, -1, NULL)) {
+    ustr = g_strdup(s);
+  } else {
+    ustr = g_locale_to_utf8(s, -1, NULL, NULL, NULL);
+  }
+  return ustr;
+}
