@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#include "common.h"
 #include "object.h"
 #include "gra.h"
 #include "mathfn.h"
@@ -89,7 +88,6 @@ static void
 get_dx_dy(int rx, int ry, int angle,
 	  double *x, double *y, double *dx, double *dy)
 {
-  double a, b;
   *x = rx * cos(- angle * MPI / 18000);
   *y = ry * sin(- angle * MPI / 18000);
   if (*y == 0) {
@@ -100,6 +98,7 @@ get_dx_dy(int rx, int ry, int angle,
       *dy = -1;
     }
   } else {
+    double a, b;
     a = 1.0 * ry * ry * (*x) / rx / rx / (*y);
     b = 1 / sqrt(1 + a * a);
     if (*y < 0) {
@@ -266,7 +265,6 @@ arcdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     draw_marker(obj, inst, GC, join, miter, width, x, y, rx, ry, angle1, angle2, fr, fg, fb, fa);
   }
 
-  GRAaddlist(GC,obj,inst,(char *)argv[0],(char *)argv[1]);
   return 0;
 }
 
@@ -289,9 +287,7 @@ arcgeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
   } else if (strcmp(field, "angle1") == 0){
     if (val < 0) {
       val %= 36000;
-      if (val < 0) {
-	val += 36000;
-      }
+      val += 36000;
     } else if (val > 36000) {
       val %= 36000;
     }
@@ -490,7 +486,7 @@ arcchange(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
     break;
   case ARC_POINT_TYPE_ANGLE1:
   case ARC_POINT_TYPE_ANGLE2:
-    ret = _putobj(obj, "angle1", inst, &a1);
+    _putobj(obj, "angle1", inst, &a1);
     ret = _putobj(obj, "angle2", inst, &a2);
     break;
   default:
@@ -631,7 +627,7 @@ arcflip(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 static int
 arczoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  int i,snum,*sdata,rx,ry,x,y,refx,refy,width,preserve_width;
+  int snum,*sdata,rx,ry,x,y,refx,refy,width,preserve_width;
   double zoom_x, zoom_y;
   struct narray *style;
 
@@ -661,6 +657,7 @@ arczoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     ry = 1;
 
   if (! preserve_width) {
+    int i;
     double zoom;
     zoom = MIN(zoom_x, zoom_y);
     width=width*zoom;
