@@ -37,6 +37,7 @@
 #include "oroot.h"
 #include "ofit.h"
 #include "odata.h"
+#include "shell.h"
 
 #include "math/math_equation.h"
 
@@ -204,18 +205,23 @@ fitequation(struct objlist *obj,N_VALUE *inst,N_VALUE *rval, int argc,char **arg
 }
 
 static MathEquation *
-create_math_equation(struct objlist *obj, const char *math, const char *field)
+create_math_equation(struct objlist *obj, char *math, const char *field)
 {
   MathEquation *code;
-  int rcode;
+  int rcode, security;
+  enum EOEQ_ASSIGN_TYPE type;
 
-  code = ofile_create_math_equation(NULL, 2, FALSE, FALSE, FALSE, FALSE, TRUE);
+  security = get_security();
+  type = (security) ? EOEQ_ASSIGN_TYPE_BOTH : EOEQ_ASSIGN_TYPE_ASSIGN;
+  code = ofile_create_math_equation(NULL, type, 2, FALSE, FALSE, FALSE, FALSE, TRUE);
   if (code == NULL) {
     return NULL;
   }
 
   rcode = math_equation_parse(code, math);
-  if (rcode) {
+  if (rcode == 0 && code->use_eoeq_assign) {
+    replace_eoeq_token(math);
+  } else if (rcode) {
     show_eqn_error(obj, code, math, field, rcode);
     math_equation_free(code);
     return NULL;
@@ -284,7 +290,8 @@ fit_put_weight_func(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
 {
   char *math;
   MathEquation *code;
-  int rcode;
+  int rcode, security;
+  enum EOEQ_ASSIGN_TYPE type;
 
   math = argv[2];
   if (math == NULL) {
@@ -298,12 +305,16 @@ fit_put_weight_func(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
     return 0;
   }
 
-  code = ofile_create_math_equation(NULL, 3, FALSE, TRUE, FALSE, FALSE, TRUE);
+  security = get_security();
+  type = (security) ? EOEQ_ASSIGN_TYPE_BOTH : EOEQ_ASSIGN_TYPE_ASSIGN;
+  code = ofile_create_math_equation(NULL, type, 3, FALSE, TRUE, FALSE, FALSE, TRUE);
   if (code == NULL)
     return 1;
 
   rcode = math_equation_parse(code, math);
-  if (rcode) {
+  if (rcode == 0 && code->use_eoeq_assign) {
+    replace_eoeq_token(math);
+  } else if (rcode) {
     show_eqn_error(obj, code, math, argv[1], rcode);
   }
 
@@ -377,7 +388,7 @@ show_poly_equation(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe
 }
 
 static int
-show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe, int num)
+show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
 {
   int j;
   GString *info;
@@ -438,7 +449,7 @@ fitpoly(struct fitlocal *fitlocal,
 */
 {
   int i,j,k,dim;
-  double yy,y1,y2,derror,sy,correlation,wt,sum;
+  double yy,derror,sy,correlation,wt,sum;
   vector b,x1,x2,coe;
   matrix m;
 
@@ -453,6 +464,7 @@ fitpoly(struct fitlocal *fitlocal,
   }
   sum=0;
   for (k=0;k<num;k++) {
+    double y1;
     if (weight) wt=wdata[k];
     else wt=1;
     sum+=wt;
@@ -477,6 +489,7 @@ fitpoly(struct fitlocal *fitlocal,
       }
   }
   if (through) {
+    double y2;
     y2=y0;
     x2[0]=1;
     for (i=1;i<dim;i++) x2[i]=x2[i-1]*x0;
@@ -507,7 +520,7 @@ fitpoly(struct fitlocal *fitlocal,
     return FitError_Fatal;
   }
 
-  if (disp && show_poly_result(fitlocal, type, coe, derror)) {
+  if (disp && show_poly_result(fitlocal, type, coe)) {
     return FitError_Fatal;
   }
 
@@ -898,7 +911,7 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   struct fitlocal *fitlocal;
   int i,through,dimension,deriv,disp;
   enum FIT_OBJ_TYPE type;
-  double x,y,x0,y0,converge,wt;
+  double x0,y0,converge,wt;
   struct narray *darray;
   double *data,*wdata;
   char *equation,*func,prm[32];
@@ -976,6 +989,7 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   num=0;
   err2=err3=FALSE;
   for (i=0;i<dnum;i++) {
+    double x, y;
     x=data[i*2];
     y=data[i*2+1];
     if (weight) {
@@ -1117,7 +1131,7 @@ fitcalc(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   rval->d = 0;
 
-  x = * (double *) argv[2];
+  x = arg_to_double(argv, 2);
 
   _getobj(obj,"_local",inst,&fitlocal);
   if (fitlocal->result_code == NULL) {
@@ -1127,7 +1141,7 @@ fitcalc(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       return 0;
     }
 
-    eq = ofile_create_math_equation(NULL, 0, FALSE, FALSE, FALSE, FALSE, TRUE);
+    eq = ofile_create_math_equation(NULL, EOEQ_ASSIGN_TYPE_ASSIGN, 0, FALSE, FALSE, FALSE, FALSE, TRUE);
     if (eq == NULL) {
       return 1;
     }
