@@ -75,9 +75,8 @@ oGRAinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 static int
 oGRAdisconnect(struct objlist *obj,void *inst,int clear)
 {
-  struct objlist *dobj,*gobj;
+  struct objlist *dobj;
   struct narray *sarray;
-  char *list;
   N_VALUE *dinst;
   char *device,*dfield,*gfield;
   int oid,did,gid;
@@ -85,19 +84,22 @@ oGRAdisconnect(struct objlist *obj,void *inst,int clear)
   _getobj(obj,"oid",inst,&oid);
   _getobj(obj,"_device",inst,&device);
   _putobj(obj,"_device",inst,NULL);
-  if (device!=NULL) {
-    if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
-    && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
-      if ((!chkobjfield(dobj,"_list"))
-      && (!_getobj(dobj,"_list",dinst,&sarray)) && (sarray!=NULL)) {
-        list=arraynget_str(sarray,0);
-        if (((gobj=getobjlist(list,&gid,&gfield,NULL))!=NULL)
-        && (gobj==obj) && (gid==oid) && (strcmp(gfield,"open")==0)) {
-          arrayfree2(sarray);
-          _putobj(dobj,"_list",dinst,NULL);
-          _exeobj(dobj,"disconnect",dinst,0,NULL);
-          if (clear) _exeobj(dobj,"disconnect",dinst,0,NULL);
-        }
+  if (device == NULL) {
+    return 0;
+  }
+  if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
+      && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
+    if ((!chkobjfield(dobj,"_list"))
+        && (!_getobj(dobj,"_list",dinst,&sarray)) && (sarray!=NULL)) {
+      struct objlist *gobj;
+      char *list;
+      list=arraynget_str(sarray,0);
+      if (((gobj=getobjlist(list,&gid,&gfield,NULL))!=NULL)
+          && (gobj==obj) && (gid==oid) && (strcmp(gfield,"open")==0)) {
+        arrayfree2(sarray);
+        _putobj(dobj,"_list",dinst,NULL);
+        _exeobj(dobj,"disconnect",dinst,0,NULL);
+        if (clear) _exeobj(dobj,"disconnect",dinst,0,NULL);
       }
     }
   }
@@ -148,17 +150,14 @@ close_gc(struct objlist *obj, N_VALUE *inst, int GC)
 static int
 oGRAopen(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  int anum,id;
   struct narray iarray;
   int GC, r;
   char *device,*dev,*gfield;
-  struct objlist *dobj,*gobj,*robj;
-  N_VALUE *dinst,*ginst;
+  struct objlist *dobj,*robj;
+  N_VALUE *dinst;
   void *local;
-  struct narray **list;
   int oid,gid;
   int topm,leftm,width,height,zoom;
-  int output,charheight,chardescent,strwidth;
 
   _getobj(obj,"device",inst,&device);
   _getobj(obj,"GC",inst,&GC);
@@ -178,6 +177,9 @@ oGRAopen(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       return 1;
     }
   } else {
+    int anum,id;
+    struct narray **list;
+    int output, charheight, chardescent, strwidth;
     arrayinit(&iarray,sizeof(int));
     if (getobjilist(device,&dobj,&iarray,FALSE,NULL)) return 1;
     anum=arraynum(&iarray);
@@ -247,6 +249,8 @@ oGRAopen(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
     /* clear gra connected to target device */
     if ((list!=NULL) && (*list!=NULL) && (arraynum(*list)!=0)) {
+      struct objlist *gobj;
+      N_VALUE *ginst;
       if (((gobj=getobjlist(arraynget_str(*list,0),&gid,&gfield,NULL))!=NULL)
       && ((ginst=chkobjinstoid(gobj,gid))!=NULL)
       && (!_getobj(gobj,"_device",ginst,&dev))) {
@@ -297,11 +301,12 @@ oGRAredraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   field=(char *)(argv[1]);
   _getobj(obj,"oid",inst,&oid);
   _getobj(obj,"_device",inst,&device);
-  if (device!=NULL) {
-    if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
-    && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
-      if (chkobjfield(dobj,field)==0) _exeobj(dobj,field,dinst,0,NULL);
-    }
+  if (device == NULL) {
+    return 0;
+  }
+  if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
+      && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
+    if (chkobjfield(dobj,field)==0) _exeobj(dobj,field,dinst,0,NULL);
   }
   return 0;
 }
@@ -320,11 +325,12 @@ oGRAclear(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   if (GC!=-1) GRAreopen(GC);
   _getobj(obj,"oid",inst,&oid);
   _getobj(obj,"_device",inst,&device);
-  if (device!=NULL) {
-    if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
-    && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
-      if (chkobjfield(dobj,field)==0) _exeobj(dobj,field,dinst,0,NULL);
-    }
+  if (device == NULL) {
+    return 0;
+  }
+  if (((dobj=getobjlist(device,&did,&dfield,NULL))!=NULL)
+      && ((dinst=chkobjinstoid(dobj,did))!=NULL)) {
+    if (chkobjfield(dobj,field)==0) _exeobj(dobj,field,dinst,0,NULL);
   }
   return 0;
 }
@@ -386,10 +392,7 @@ oGRAdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   int GC, layer;
   struct objlist *draw;
   struct narray *array;
-  char **drawrable;
-  const char *objname;
   char *oGRAargv[2];
-  int j,i,anum,instnum;
 
   _getobj(obj,"GC",inst,&GC);
   if (GC==-1) {
@@ -405,6 +408,9 @@ oGRAdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     if ((draw=getobject("draw"))==NULL) return 1;
     oGRAdrawparent(draw, oGRAargv, layer);
   } else {
+    char **drawrable;
+    const char *objname;
+    int j,i,anum,instnum;
     anum=arraynum(array);
     drawrable=arraydata(array);
     for (j=0;j<anum;j++) {
