@@ -290,10 +290,9 @@ draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata
 {
   int width, fr, fg, fb, fa, headlen, headwidth;
   int join, miter, head_begin, head_end;
-  int x, y, x0, y0, x1, y1, x2, y2, x3, y3, type;
+  int x0, y0, x1, y1, x2, y2, x3, y3, type;
   struct narray *style;
   int snum, *sdata;
-  int i;
   int ap[6], ap2[6];
 
   _getobj(obj, "stroke_R", inst, &fr);
@@ -341,6 +340,7 @@ draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata
   if (num > 2 && close_path) {
     GRAdrawpoly(GC, num, pdata, GRA_FILL_MODE_NONE);
   } else {
+    int x, y, i;
     x = points2[0];
     y = points2[1];
 
@@ -379,7 +379,7 @@ arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   int GC, w, h, intp, i, j, num, close_path;
   struct narray *points;
   int *points2, *pdata;
-  int x0, y0, x1, y1, type, stroke, fill, clip, zoom;
+  int x1, y1, type, stroke, fill, clip, zoom;
 
   if (_exeparent(obj, argv[1], inst, rval, argc, argv)) {
     return 1;
@@ -424,6 +424,7 @@ arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   j = 0;
   x1 = y1 = 0;
   for (i = 0; i < num; i++) {
+    int x0, y0;
     x0 = pdata[2 * i];
     y0 = pdata[2 * i + 1];
     if (i == 0 || x0 != x1 || y0 != y1) {
@@ -452,7 +453,6 @@ arrowdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   }
 
   g_free(points2);
-  GRAaddlist(GC, obj, inst, argv[0], argv[1]);
   return 0;
 }
 
@@ -787,30 +787,31 @@ curvematch(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
 
   if (minx == maxx && miny == maxy) {
     rval->i = point_match(obj, inst, type, fill, err, minx, miny);
-  } else {
-    if (_exeobj(obj, "bbox", inst, 0, NULL)) {
-      return 1;
-    }
-    _getobj(obj, "bbox", inst, &array);
+    return 0;
+  }
 
-    if (array == NULL) {
-      return 0;
-    }
+  if (_exeobj(obj, "bbox", inst, 0, NULL)) {
+    return 1;
+  }
+  _getobj(obj, "bbox", inst, &array);
 
-    if (arraynum(array) < 4) {
-      return 1;
-    }
+  if (array == NULL) {
+    return 0;
+  }
 
-    bminx = arraynget_int(array, 0);
-    bminy = arraynget_int(array, 1);
-    bmaxx = arraynget_int(array, 2);
-    bmaxy = arraynget_int(array, 3);
-    if (minx <= bminx && bminx <= maxx &&
-	minx <= bmaxx && bmaxx <= maxx &&
-	miny <= bminy && bminy <= maxy &&
-	miny <= bmaxy && bmaxy <= maxy) {
-      rval->i = TRUE;
-    }
+  if (arraynum(array) < 4) {
+    return 1;
+  }
+
+  bminx = arraynget_int(array, 0);
+  bminy = arraynget_int(array, 1);
+  bmaxx = arraynget_int(array, 2);
+  bmaxy = arraynget_int(array, 3);
+  if (minx <= bminx && bminx <= maxx &&
+      minx <= bmaxx && bmaxx <= maxx &&
+      miny <= bminy && bminy <= maxy &&
+      miny <= bmaxy && bmaxy <= maxy) {
+    rval->i = TRUE;
   }
   return 0;
 }
@@ -840,10 +841,11 @@ static int
 curve_move(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct narray *points;
-  int i, num, *pdata, type;
+  int type;
 
   _getobj(obj, "type", inst, &type);
   if (type  == PATH_TYPE_CURVE) {
+    int i, num, *pdata;
     _getobj(obj, "_points", inst, &points);
     num = arraynum(points);
     pdata = arraydata(points);
@@ -962,7 +964,7 @@ arrowput_arrow(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
   return 0;
 }
 
-static struct objtable arrow[] = {
+static struct objtable path_obj[] = {
   {"init", NVFUNC, NEXEC, arrowinit, NULL, 0},
   {"done", NVFUNC, NEXEC, arrowdone, NULL, 0},
   {"next", NPOINTER, 0, NULL, NULL, 0},
@@ -1020,11 +1022,11 @@ static struct objtable arrow[] = {
   {"arrow", NENUM, NWRITE, arrowput_arrow, arrowchar, 0},
 };
 
-#define TBLNUM (sizeof(arrow) / sizeof(*arrow))
+#define TBLNUM (sizeof(path_obj) / sizeof(*path_obj))
 
 void *
 addpath(void)
 /* addarrow() returns NULL on error */
 {
-  return addobject(NAME, ALIAS, PARENT, OVERSION, TBLNUM, arrow, ERRNUM, patherrorlist, NULL, NULL);
+  return addobject(NAME, ALIAS, PARENT, OVERSION, TBLNUM, path_obj, ERRNUM, patherrorlist, NULL, NULL);
 }
