@@ -142,10 +142,10 @@ addfontcontrol(char *s,int *po,int *fchange,int *jchange,
                     int *fff,int *ffb,int *ffj,int script)
 {
   int j;
-  char *style;
 
   j=*po;
   if (fchange[script]) {
+    char *style;
     j+=sprintf(s+j,"%%F{%s}",fontchar[fff[script]]);
     switch (ffb[script]) {
     case FONT_STYLE_NORMAL:
@@ -232,14 +232,14 @@ remarkconv(char *str,int ff,int fj,int fb,int *fnameid,char *prmfile)
         j+=7;
       } else if ((toupper(s[i+1])=='F') && (strchr("THCthc",s[i+2])!=NULL)) {
         fchange[script]=TRUE;
-        if (toupper(s[i+2]=='T')) fff[script]=FONT_TYPE_SERIF;
-        else if (toupper(s[i+2]=='H')) fff[script]=FONT_TYPE_SANS_SERIF;
-        else if (toupper(s[i+2]=='C')) fff[script]=FONT_TYPE_MONOSPACE;
+        if (toupper(s[i+2])=='T') fff[script]=FONT_TYPE_SERIF;
+        else if (toupper(s[i+2])=='H') fff[script]=FONT_TYPE_SANS_SERIF;
+        else if (toupper(s[i+2])=='C') fff[script]=FONT_TYPE_MONOSPACE;
         i+=2;
       } else if ((toupper(s[i+1])=='J') && (strchr("GMgm",s[i+2])!=NULL)) {
         jchange[script]=TRUE;
-        if (toupper(s[i+2]=='M')) ffj[script]=0;
-        else if (toupper(s[i+2]=='G')) ffj[script]=1;
+        if (toupper(s[i+2])=='M') ffj[script]=0;
+        else if (toupper(s[i+2])=='G') ffj[script]=1;
         i+=2;
       } else if (toupper(s[i+1])=='R') {
         fchange[script]=TRUE;
@@ -269,7 +269,7 @@ remarkconv(char *str,int ff,int fj,int fb,int *fnameid,char *prmfile)
         addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
         j+=sprintf(s2+j,"%%{system:0:time:2}");
         i++;
-      } else if (s[i+1]=='t') {
+      } else if (s[i+1]=='T') {
         addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
         j+=sprintf(s2+j,"%%{system:0:time:3}");
         i++;
@@ -303,7 +303,7 @@ remarkconv(char *str,int ff,int fj,int fb,int *fnameid,char *prmfile)
           addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
           j+=sprintf(s2+j,"%%{file:%d:time:4}",fnameid[file]);
           i+=3;
-        } else if (s[i+3]=='t') {
+        } else if (s[i+3]=='T') {
           addfontcontrol(s2,&j,fchange,jchange,fff,ffb,ffj,script);
           j+=sprintf(s2+j,"%%{system:%d:time:3}",fnameid[file]);
           i+=3;
@@ -337,85 +337,41 @@ remarkconv(char *str,int ff,int fj,int fb,int *fnameid,char *prmfile)
 
 static char *
 mathconv(char *math)
-/* INTEG ---> SUM
-   F(,) ---> F(,,)
+/* INTEG ---> sum
    NAN ---> CONT
-   UNDEF ---> BREAK
+   NONE ---> BREAK
+   =  ---> ;
 */
 {
-  int i,j;
-  int f,g,fb[50],gb[50];
+  int i;
   char *m;
+  GString *new_math;
 
-  if ((m=g_malloc(strlen(math)+100))==NULL) return NULL;
-  j=0;
-  f=g=0;
-  for (i=0;math[i]!='\0';i++) {
-    if (strncmp(math+i,"NAN",3)==0) {
-      strcpy(m+j,"CONT");
-      i+=2;
-      j+=4;
-    } else if (strncmp(math+i,"NONE",3)==0) {
-      strcpy(m+j,"BREAK");
-      i+=3;
-      j+=5;
-    } else if (strncmp(math+i,"INTEG",5)==0) {
-      strcpy(m+j,"SUM");
-      i+=4;
-      j+=3;
-    } else if (strncmp(math+i,"DIF(",4)==0) {
-      strcpy(m+j,"DIF");
-      i+=2;
-      j+=3;
-    } else if (strncmp(math+i,"IF(",3)==0) {
-      strcpy(m+j,"IF");
-      i++;
-      j+=2;
-    } else if (strncmp(math+i,"LOG(",4)==0) {
-      strcpy(m+j,"LOG");
-      i+=2;
-      j+=3;
-    } else if (strncmp(math+i,"F(",2)==0) {
-      f++;
-      fb[f]=0;
-    } else if (strncmp(math+i,"G(",2)==0) {
-      g++;
-      gb[g]=0;
-    } else if (math[i]=='(') {
-      if (f) fb[f]++;
-      if (g) gb[f]++;
-      m[j]=math[i];
-      j++;
-    } else if (math[i]==')') {
-      if (f) {
-        fb[f]--;
-        if (fb[f]==0) {
-          m[j]=',';
-          m[j+1]='0';
-          f--;
-          j+=2;
-        }
-      }
-      if (g) {
-        gb[g]--;
-        if (gb[g]==0) {
-          m[j]=',';
-          m[j+1]='0';
-          g--;
-          j+=2;
-        }
-      }
-      m[j]=math[i];
-      j++;
-    } else if ((math[i]!=' ') && (math[i]!='\t')) {
-      m[j]=math[i];
-      j++;
+  new_math = g_string_new("");
+  if (new_math == NULL) {
+    return NULL;
+  }
+  for (i = 0; math[i]; i++) {
+    if (strncmp(math + i, "NAN", 3) == 0) {
+      g_string_append(new_math, "CONT");
+      i += 2;
+    } else if (strncmp(math + i, "NONE", 4) == 0) {
+      g_string_append(new_math, "BREAK");
+      i += 3;
+    } else if (strncmp(math + i, "INTEG", 5) == 0) {
+      g_string_append(new_math, "sum");
+      i += 4;
+    } else if (math[i] == '=') {
+      g_string_append(new_math, ";\n");
+    } else if ((math[i] != ' ') && (math[i] != '\t')) {
+      g_string_append_c(new_math, math[i]);
     }
   }
-  m[j]='\0';
-  if (m[0]=='\0') {
-    g_free(m);
-    m=NULL;
+  if (new_math->len == 0) {
+    g_string_free(new_math, TRUE);
+    m = NULL;
+  } else {
+    m = g_string_free(new_math, FALSE);
   }
   return m;
 }
@@ -438,8 +394,6 @@ sscanf2(char *buffer,char *format,...)
 {
   va_list ap;
   int i,num;
-  int *d;
-  double *e;
   char *s;
   char *endptr;
 
@@ -449,12 +403,14 @@ sscanf2(char *buffer,char *format,...)
   i=0;
   while (format[i]!='\0') {
     if (format[i]=='d') {
+      int *d;
       d=va_arg(ap,int *);
       *d=strtol(s,&endptr,10);
       num++;
       if (endptr[0]=='\0') break;
       s=endptr;
     } else if (format[i]=='e') {
+      double *e;
       e=va_arg(ap,double *);
       *e=strtod(s,&endptr);
       num++;
