@@ -21,6 +21,8 @@
  *
  */
 
+#include "common.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -28,13 +30,10 @@
 #include <ctype.h>
 #include <glib.h>
 
-#include "common.h"
-
 #include "object.h"
 #include "gra.h"
 #include "oroot.h"
 #include "odraw.h"
-#include "otext.h"
 #include "olegend.h"
 #include "mathfn.h"
 #include "nstring.h"
@@ -90,7 +89,7 @@ textinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
   pt=DEFAULT_FONT_PT;
-  scriptsize=7000;
+  scriptsize=DEFAULT_SCRIPT_SIZE;
   if (_putobj(obj,"pt",inst,&pt)) return 1;
   if (_putobj(obj,"script_size",inst,&scriptsize)) return 1;
 
@@ -125,10 +124,10 @@ textgeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
       *(int *)(argv[2])=TEXT_SIZE_MIN;
     }
   } else if (strcmp(field,"script_size")==0) {
-    if (*(int *)(argv[2]) < TEXT_OBJ_SCRIPT_SIZE_MIN) {
-      *(int *)(argv[2])=TEXT_OBJ_SCRIPT_SIZE_MIN;
-    } else if (*(int *)(argv[2])>TEXT_OBJ_SCRIPT_SIZE_MAX) {
-      *(int *)(argv[2])=TEXT_OBJ_SCRIPT_SIZE_MAX;
+    if (*(int *)(argv[2]) < SCRIPT_SIZE_MIN) {
+      *(int *)(argv[2])=SCRIPT_SIZE_MIN;
+    } else if (*(int *)(argv[2])>SCRIPT_SIZE_MAX) {
+      *(int *)(argv[2])=SCRIPT_SIZE_MAX;
     }
   } else if (strcmp(field,"style") == 0) {
     * (int *) (argv[2]) &= (GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC);
@@ -176,7 +175,6 @@ textdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   } else {
     GRAdrawtext(GC,text,font,style,pt,space,dir,scriptsize);
   }
-  GRAaddlist(GC,obj,inst,(char *)argv[0],(char *)argv[1]);
   return 0;
 }
 
@@ -187,7 +185,7 @@ textprintf(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   char **argv2;
   int argc2;
   char *format;
-  int po, arg, i, quote, r;
+  int po, arg, i, r;
   GString *ret;
   char *arg_str;
 
@@ -207,6 +205,7 @@ textprintf(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   po=0;
   arg=1;
   while (format[po]!='\0') {
+    int quote;
     quote=FALSE;
     for (i=po;(quote || (format[i]!='%')) && (format[i]!='\0');i++) {
       if (quote) {
@@ -224,6 +223,12 @@ textprintf(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       continue;
     }
 
+    if (format[po + 1] == '%') {
+      g_string_append_c(ret, '%');
+      po += 2;
+      continue;
+    }
+
     arg_str = (arg < argc2 && argv2[arg]) ? argv2[arg] : NULL;
     r = add_printf_formated_str(ret, format + po, arg_str, &i);
 
@@ -241,14 +246,11 @@ textprintf(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 static int
 textbbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  int minx,miny,maxx,maxy;
   struct narray *array;
   int x,y,pt,space,dir,scriptsize,raw,style;
   char *font;
   char *text;
-  int gx0,gy0,gx1,gy1;
-  int i,ggx[4],ggy[4];
-  double si,co;
+  int bbox[4];
 
   array=rval->array;
   if (arraynum(array)!=0) return 0;
@@ -262,36 +264,12 @@ textbbox(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   _getobj(obj,"raw",inst,&raw);
   _getobj(obj,"font",inst,&font);
   _getobj(obj,"style",inst,&style);
-  if (raw) {
-    GRAtextextentraw(text,font,style,pt,space,&gx0,&gy0,&gx1,&gy1);
-  } else {
-    GRAtextextent(text,font,style,pt,space,scriptsize,&gx0,&gy0,&gx1,&gy1,FALSE);
-  }
-  si=-sin(dir/18000.0*MPI);
-  co=cos(dir/18000.0*MPI);
-  ggx[0]=x+gx0*co-gy0*si;
-  ggy[0]=y+gx0*si+gy0*co;
-  ggx[1]=x+gx1*co-gy0*si;
-  ggy[1]=y+gx1*si+gy0*co;
-  ggx[2]=x+gx0*co-gy1*si;
-  ggy[2]=y+gx0*si+gy1*co;
-  ggx[3]=x+gx1*co-gy1*si;
-  ggy[3]=y+gx1*si+gy1*co;
-  minx=ggx[0];
-  maxx=ggx[0];
-  miny=ggy[0];
-  maxy=ggy[0];
-  for (i=1;i<4;i++) {
-    if (ggx[i]<minx) minx=ggx[i];
-    if (ggx[i]>maxx) maxx=ggx[i];
-    if (ggy[i]<miny) miny=ggy[i];
-    if (ggy[i]>maxy) maxy=ggy[i];
-  }
+  text_get_bbox(x, y, text, font, style, pt, dir, space, scriptsize, raw, bbox);
   if ((array==NULL) && ((array=arraynew(sizeof(int)))==NULL)) return 1;
-  arrayins(array,&(maxy),0);
-  arrayins(array,&(maxx),0);
-  arrayins(array,&(miny),0);
-  arrayins(array,&(minx),0);
+  arrayins(array, bbox + 3, 0);
+  arrayins(array, bbox + 2, 0);
+  arrayins(array, bbox + 1, 0);
+  arrayins(array, bbox, 0);
   if (arraynum(array)==0) {
     arrayfree(array);
     rval->array = NULL;
@@ -396,11 +374,8 @@ static int
 textmatch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int minx,miny,maxx,maxy,err;
-  int bminx,bminy,bmaxx,bmaxy;
   struct narray *array;
   int gx0,gy0,gx1,gy1;
-  int px,py,px2,py2;
-  double si,co;
   int x,y,pt,space,dir,scriptsize,raw, style;
   char *font;
   char *text;
@@ -423,6 +398,8 @@ textmatch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   maxy=*(int *)(argv[5]);
   err=*(int *)(argv[6]);
   if ((minx==maxx) && (miny==maxy)) {
+    int px,py,px2,py2;
+    double si,co;
     if (raw) {
       GRAtextextentraw(text,font,style,pt,space,&gx0,&gy0,&gx1,&gy1);
     } else {
@@ -442,6 +419,7 @@ textmatch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       rval->i=TRUE;
     }
   } else {
+    int bminx,bminy,bmaxx,bmaxy;
     if (_exeobj(obj,"bbox",inst,0,NULL)) return 1;
     _getobj(obj,"bbox",inst,&array);
     if (arraynum(array)<4) return 1;
@@ -465,37 +443,45 @@ text_set_text(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **ar
 
   str = argv[2];
 
-  if (str == NULL)
+  if (str == NULL) {
     return 0;
+  }
 
-  if (g_utf8_validate(str, -1, NULL))
+  if (g_utf8_validate(str, -1, NULL)) {
     return textgeometry(obj, inst, rval, argc, argv);
+  }
 
   ptr = sjis_to_utf8(str);
   if (ptr) {
-    g_free(str);
-    argv[2] = ptr;
-    return textgeometry(obj, inst, rval, argc, argv);
+    int r;
+    r = textgeometry(obj, inst, rval, argc, argv);
+    if (r) {
+      g_free(ptr);
+    } else {
+      g_free(str);
+      argv[2] = ptr;
+    }
+    return r;
   }
 
   ptr = g_locale_to_utf8(str, -1, NULL, &len, NULL);
   if (ptr) {
-    char *tmp;
-    g_free(str);
-    tmp = g_strdup(ptr);
-    g_free(ptr);
-    if (tmp == NULL)
-      return 1;
-
-    argv[2] = tmp;
-    return textgeometry(obj, inst, rval, argc, argv);
+    int r;
+    r = textgeometry(obj, inst, rval, argc, argv);
+    if (r) {
+      g_free(ptr);
+    } else {
+      g_free(str);
+      argv[2] = ptr;
+    }
+    return r;
   }
 
   error(obj, ERR_INVALID_STR);
   return 1;
 }
 
-static struct objtable text[] = {
+static struct objtable text_obj[] = {
   {"init",NVFUNC,NEXEC,textinit,NULL,0},
   {"done",NVFUNC,NEXEC,textdone,NULL,0},
   {"next",NPOINTER,0,NULL,NULL,0},
@@ -527,15 +513,14 @@ static struct objtable text[] = {
   {"jfont",NSTR,NWRITE,textgeometry,NULL,0},
 };
 
-#define TBLNUM (sizeof(text) / sizeof(*text))
+#define TBLNUM (sizeof(text_obj) / sizeof(*text_obj))
 
 void *
 addtext()
 /* addtext() returns NULL on error */
 {
-  unsigned int i;
-
   if (TextConfigHash == NULL) {
+    unsigned int i;
     TextConfigHash = nhash_new();
     if (TextConfigHash == NULL)
       return NULL;
@@ -548,5 +533,5 @@ addtext()
     }
   }
 
-  return addobject(NAME,NULL,PARENT,OVERSION,TBLNUM,text,ERRNUM,texterrorlist,NULL,NULL);
+  return addobject(NAME,NULL,PARENT,OVERSION,TBLNUM,text_obj,ERRNUM,texterrorlist,NULL,NULL);
 }
